## Filename    : chart_view.py
## Author(s)   : Michel Le Borgne
## Created     : 4/3/2010
## Revision    : 
## Source      : 
##
## Copyright 2009 - 2010 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided hereunder is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE 
##     
##
## Contributor(s): Geoffroy Andrieux, Nolwenn Le Meur
##
"""
Main views for the Cadbiom gui
"""
import gtk

GRAPH_BG_COLOR = "#FEFAFA"

class ChartView(gtk.DrawingArea):
    """
    Class for the main drawing window of charter
    """
    def __init__(self, width, height, drawing_style, controler, model):
        """
        @param width, height: int - size of the view
        @param drawing_style: a Drawing style associated with the pattern 
        @param controler: the controller as in the MVC pattern
        @param model: a ChartModel    
        """
        gtk.DrawingArea.__init__(self)
        self.controler = controler
        controler.set_view(self)
        self.model = model

        # attach as model observer
        model.attach(self)
        
        # signals
        self.set_events(gtk.gdk.EXPOSURE_MASK
                             | gtk.gdk.LEAVE_NOTIFY_MASK
                             | gtk.gdk.BUTTON_PRESS_MASK
                             | gtk.gdk._2BUTTON_PRESS
                             | gtk.gdk.BUTTON_RELEASE_MASK
                             | gtk.gdk.POINTER_MOTION_MASK
                             | gtk.gdk.POINTER_MOTION_HINT_MASK)
        self.connect("expose-event", self.expose_event)
        self.connect("configure_event", self.configure_event)
        
        # controlled events
        self.connect("button_press_event", controler.button_press)
        self.connect("button_release_event", controler.button_release)
        self.connect("motion_notify_event", controler.motion_notify)
        
        # drawing stuffs
        drawing_style.view = self
        self.drawing_style = drawing_style
        controler.drawing_style = drawing_style
        self.draw_width = width       # drawing window
        self.min_width = width
        self.draw_height = height
        self.min_height = height
        self.set_size_request(width, height)
        self.stylepango = self.create_pango_layout("")
        self.pixmap = None # cannot be initialized before self.window creation
        self.page = None
        self.zoom_count = 0
        
    def expose_event(self, widget, event):
        """
        Redisplay drawing after cover and exposure of the window
        """

        self.pixmap.draw_rectangle(self.get_style().white_gc,
                          True, 0, 0, self.draw_width, self.draw_height)
        # draw in the pixmap
        self.model.draw(self)
        # expose the drawing
        self.window.draw_drawable(self.get_style().fg_gc[gtk.STATE_NORMAL],
                                self.pixmap, 0, 0, 0, 0, 
                                self.draw_width, self.draw_height)
        if self.page:
            self.page.notify(widget)
        return True

    def configure_event(self, widget, event):
        """
        Initialisation and change in size
        """       
        # adjust to visible window 
        xor , yor, width, height = widget.get_allocation()
        self.draw_width = max(width, self.min_width)
        self.draw_height = max(height, self.min_height)
        # set pixmap if necesssary
        if not self.pixmap:
            self.pixmap = gtk.gdk.Pixmap(self.window, 
                                         self.draw_width, self.draw_height) 
        # adjust the pixmap to size
        size = self.pixmap.get_size()
        if size[0] != self.draw_width or size[1] != self.draw_height:
            self.set_size_request(self.draw_width, self.draw_height)
            self.pixmap = gtk.gdk.Pixmap(self.window, 
                                         self.draw_width, self.draw_height)   
        return True
        
    def update(self):
        """
        Display a new drawing of the model
        """
        # update only visible views
        if not self.window:
            return

        # resize if virtual drawing window size changed 
        size = self.pixmap.get_size()
        if size[0] != self.draw_width or size[1] != self.draw_height:
            self.set_size_request(self.draw_width, self.draw_height)
            self.pixmap = gtk.gdk.Pixmap(self.window, 
                                         self.draw_width, self.draw_height)
        self.pixmap.draw_rectangle(self.get_style().white_gc,
                          True, 0, 0, self.draw_width, self.draw_height)
        # draw in the pixmap
        self.model.draw(self)
        # expose the drawing
        self.window.draw_drawable(self.get_style().fg_gc[gtk.STATE_NORMAL],
                                self.pixmap, 0, 0, 0, 0, 
                                self.draw_width, self.draw_height)


    def zoom_minus(self):
        """
        Zoom out function
        """
        if self.zoom_count < -10:
            return
        self.min_height = int(self.min_height * 0.9)
        self.draw_height = self.min_height
        self.min_width = int(self.min_width * 0.9)
        self.draw_width = self.min_width
        self.update()
        if self.page:
            self.page.notify(None)
        self.zoom_count = self.zoom_count - 1
        
    def zoom_plus(self):
        """
        Zoom in function
        """
        if self.zoom_count > 15:
            return
        self.min_height = int(self.min_height * 1.1)
        self.draw_height = self.min_height
        self.min_width = int(self.min_width * 1.1)
        self.draw_width = self.min_width
        self.update()
        if self.page:
            self.page.notify(None)
        self.zoom_count += 1

class NavView(ChartView):
    """
    Specialized ChartView for navigator
    """
    def __init__(self, width, height, drst, controler, model):
        """
        same parameters than in upper class constructor
        """
        ChartView.__init__(self, width, height, drst, controler, model)
        self.x_ret = 0
        self.y_ret = 0
        self.w_ret = 1
        self.h_ret = 1
        self.show_all()
        
    def configure_event(self, widget, event):
        """
        Initialization and change in size
        """
        xor, yor, width, height = widget.get_allocation()
        self.draw_width = width
        self.draw_height = height        
        
        self.pixmap = gtk.gdk.Pixmap(self.window, 
                                     self.draw_width, self.draw_height)
        self.pixmap.draw_rectangle(widget.get_style().white_gc,
                          True, 0, 0, self.draw_width, self.draw_height) 
        self.ret_gc = self.window.new_gc()
        self.ret_gc.set_rgb_fg_color(gtk.gdk.color_parse('red'))         
        return True
        
    def expose_event(self, widget, event):
        """
        Redisplay drawing after cover and exposure of the window
        """
        self.pixmap.draw_rectangle(self.get_style().white_gc,
                          True, 0, 0, self.draw_width, self.draw_height)
        # draw in the pixmap
        self.model.draw(self)
        # expose the drawing
        self.window.draw_drawable(self.get_style().fg_gc[gtk.STATE_NORMAL],
                                self.pixmap, 0, 0, 0, 0, 
                                self.draw_width, self.draw_height)
        # draw reticule
        self.window.draw_rectangle(self.ret_gc,
                          False, self.x_ret, self.y_ret, 
                          self.w_ret, self.h_ret)        
        return True
        
    def update(self):
        """
        Display a new drawing of the model
        """
        # update only visible views
        if not self.window:
            return
        # hack for notebook
        if not self.pixmap:
            return
        # draw in the pixmap
        self.pixmap.draw_rectangle(self.get_style().white_gc,
                          True, 0, 0, self.draw_width, self.draw_height)
        self.model.draw(self)
        # expose the drawing
        self.window.draw_drawable(self.get_style().fg_gc[gtk.STATE_NORMAL],
                                self.pixmap, 0, 0, 0, 0,
                                self.draw_width, self.draw_height)
        # draw reticule
        self.window.draw_rectangle(self.ret_gc, False, 
                                   self.x_ret, self.y_ret, 
                                   self.w_ret, self.h_ret)
        
    def scroll_update(self, xor, yor, width, height):
        """
        Updating function used when the linked window is scrolled
        """
        if not self.window: 
            return
        self.x_ret = int(xor * self.draw_width)
        self.y_ret = int(yor * self.draw_height)
        self.w_ret = int(width * self.draw_width)
        self.h_ret = int(height * self.draw_height)
        # expose the drawing
        self.window.draw_rectangle(self.get_style().white_gc,
                          True, 0, 0, self.draw_width, self.draw_height)
        self.window.draw_drawable(self.get_style().fg_gc[gtk.STATE_NORMAL],
                                self.pixmap, 0, 0, 0, 0, 
                                self.draw_width, self.draw_height)
        # draw reticule
        self.window.draw_rectangle(self.ret_gc, False, 
                                   self.x_ret, self.y_ret, 
                                   self.w_ret, self.h_ret)               

    def clear(self):
        """
        As it says
        """
        self.window.draw_rectangle(self.get_style().white_gc,
                          True, 0, 0, self.draw_width, self.draw_height) 


class ChartPage(gtk.Frame):
    """
    A simple graph editor for pages: a graph area wrapped in a scroll window
    """
    def __init__(self, width, height, drst, controler, model):
        """
        
        """
        gtk.Frame.__init__(self)
        # for observers
        self.__observer_list = []
        # chart view and model
        self.draw = ChartView(width, height, drst, controler, model)
        self.draw.page = self

        # wrap in a scroll window
        scr_w = gtk.ScrolledWindow()
        self.scr_w = scr_w
        self.vadj = scr_w.get_vadjustment()
        self.hadj = scr_w.get_hadjustment()
        self.hadj_h_id = self.hadj.connect("value_changed", self.notify)
        self.vadj_h_id = self.vadj.connect("value_changed", self.notify)
        self.allow_adj = True
        scr_w.set_policy(gtk.POLICY_ALWAYS, gtk.POLICY_ALWAYS)
        viewport = gtk.Viewport()
        scr_w.add(viewport)
        viewport.add(self.draw)
        self.add(scr_w)
        self.show_all()
        
    
    # observer pattern methods
    def attach(self, obs):
        """
        standard in observer pattern
        """
        if not obs in self.__observer_list:
            self.__observer_list.append(obs)
            
    def detach(self, obs):
        """
        standard in observer pattern
        """
        self.__observer_list.remove(obs)
        
    def notify(self, widget):
        """
        used when the viewport is moved from inside (hbar and vbar)
        """
        if self.allow_adj:
            x_pos = float(self.hadj.value)/self.draw.draw_width
            y_pos = float(self.vadj.value)/self.draw.draw_height
            width = self.hadj.page_size/self.draw.draw_width
            height = self.vadj.page_size/self.draw.draw_height
            for obs in self.__observer_list:
                obs.scroll_update(x_pos, y_pos, width, height)
        
    # signal call back
    def update(self, dxx, dyy):
        """
        Used when the viewport is moved from outside (navigator)
        """
        d_hadj = dxx * self.draw.draw_width
        d_vadj = dyy * self.draw.draw_height
        self.allow_adj = False
        self.hadj.value = int(self.hadj.value + d_hadj)
        self.vadj.value = int(self.vadj.value + d_vadj)
        self.scr_w.set_hadjustment(self.hadj)
        self.scr_w.set_vadjustment(self.vadj)
        self.allow_adj = True
        self.notify(None)
        


    
        



##
## Filename    : fileHandling.py
## Author(s)   : Michel Le Borgne
## Created     : 2/2/2012
## Revision    : 
## Source      : 
##
## Copyright 2009 - 2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided hereunder is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE 
##     
##
##     http:
##     mailto:
##FILE_CHOOSER_ACTION_SAVE
## Contributor(s): Geoffroy Andrieux
##

from gtk import FileChooserDialog, FileFilter, \
    STOCK_CANCEL, RESPONSE_CANCEL, RESPONSE_OK, \
    FILE_CHOOSER_ACTION_SAVE, STOCK_SAVE, \
    FILE_CHOOSER_ACTION_OPEN, STOCK_OPEN


class FileChooser(FileChooserDialog):
    """
    Create a customed file chooser
    """
    def __init__(self, title, filter_name, filter_pattern, save_window=True):
        """Open a custom file chooser.

        if save_window is False, an open window will be initialized
        """
        if save_window:
            FileChooserDialog.__init__(
                self, title, None, FILE_CHOOSER_ACTION_SAVE,
                (STOCK_CANCEL, RESPONSE_CANCEL, STOCK_SAVE, RESPONSE_OK)
            )
        else:
            FileChooserDialog.__init__(
                self, title, None, FILE_CHOOSER_ACTION_OPEN,
                (STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OPEN, RESPONSE_OK)
            )

        self.set_default_response(RESPONSE_OK)
                       
        #add a filter to see only files for BioSignal (*.sig)
        filter = FileFilter()
        filter.set_name(filter_name)
        filter.add_pattern(filter_pattern)
        self.add_filter(filter)
        
        #add a filter to see all
        no_filter = FileFilter()
        no_filter.set_name("all")
        no_filter.add_pattern("*")
        self.add_filter(no_filter)
        
    def do_action(self, action):
        """
        Action following file selection
        """
        response = self.run()
        if response == RESPONSE_OK:
            action(self.get_filename())
        elif response == RESPONSE_CANCEL:
            pass
        self.destroy()



## Filename    : edit_mvc.py
## Author(s)   : Geoffroy Andrieux
## Created     : 03/2010
## Revision    :
## Source      :
##
## Copyright 2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided here under is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Geoffroy Andrieux.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE
##
##
## Contributor(s): Michel Le Borgne, Nolwenn Le Meur
##
"""
Class for managing a triple (Model, View, Controler) associated with a model
"""

from cadbiom.models.guard_transitions.chart_model import ChartModel
from chart_view import ChartPage, NavView
from cadbiom_gui.gt_gui.graphics.drawing_style import PlainDrawing, NavDrawing
from cadbiom_gui.gt_gui.chart_controler import ChartControler, NavControler
from layout import LayoutVisitor

class EditMVC(object):
    """
    Editor object composed of a MVC pattern
    """
    def __init__(self, chart, model_name, model = None, layout = False):
        """
        @param chart: the charter using the edit mvc
        """
        self.charter = chart
        self.ident = chart.ei_id
        chart.ei_id = chart.ei_id + 1
        if model:
            self.model = model
        else:
            self.model = ChartModel(model_name)
        self.controler = ChartControler(self.model, chart.clipboard)
        xor, yor, width, height = chart.graph_notebook.get_allocation()
        # adjustment for notebook (-6, -38)
        self.viewpage = ChartPage( width - 6, height - 38, PlainDrawing(),
                                   self.controler, self.model)
        self.view = self.viewpage.draw
        self.title = self.model.name
        # layout?
        if layout:
            # default layout if necessary
            lvi = LayoutVisitor(self.view, 'hierarchical_LR')
            model.accept(lvi)
        # overview navigator
        self.navcontroler = NavControler()
        xor, yor, width, height = chart.overview_window.get_allocation()
        self.nav_view = NavView( width, height, NavDrawing(),
                                 self.navcontroler, self.model)
        self.viewpage.attach(self.nav_view) # linked to view
        self.navcontroler.attach(self.viewpage)

        # registration of auxiliary windows
        self.aux_win = []
        self.has_display = False

    def change_model(self, new_model):
        """
        Change the model component
        """
        self.clean_subwin()
        self.view.model.detach(self.view)
        # new model
        self.model = new_model
        self.controler.model = self.model
        self.navcontroler.model = self.model
        self.view.model = self.model
        self.nav_view.model = self.model
        self.view.model.attach(self.view)
        self.nav_view.model.attach(self.nav_view)
        self.display()

    def display(self):
        """
        Update and show
        """
        self.view.update()
        self.nav_view.update()
        self.viewpage.notify(None)

    def update(self, node, transition):
        """
        Used in the observer pattern
        """
        self.node = node
        self.transition = transition
        self.view.update()
        self.nav_view.update()
        self.viewpage.notify(None)

    def connect(self, chart):
        """
        When we switch from another EditMVC we must connect the buttons of the
        graphical interface to the components of the EditMVC
        """
        # connect buttons to emvc controler
        chart.drawing_button_connect()
        #self.tab_button.set_sensitive(True)
        # attach charter to new edit_MVC controler
        self.controler.attach("edit_node", chart)
        chart.chart_info.set_controler(self.controler)
        # nav window
        chart.overview_window.get_child().add(self.nav_view)

    def disconnect(self, chart, w_destroy):
        """
        Inverse operation than connect
        """
        if w_destroy:
            chart.drawing_button_disconnect()
            chart.drawing_button_unable()
        else:
            chart.drawing_button_disable()
        self.controler.detach("edit_node", chart)
        #self.tab_button.set_sensitive(False)
        if w_destroy:
            self.clean_subwin()
        chart.overview_window.get_child().remove(self.nav_view)

    def zoom_plus(self, widget):
        """
        Zoom in
        """
        self.view.zoom_plus()

    def zoom_minus(self, widget):
        """
        Zoom out
        """
        self.view.zoom_minus()


    # sub windows management
    def win_register(self, win):
        """
        Subordinate windows registration
        """
        self.aux_win.append(win)

    def win_remove(self, win):
        """
        Subordinate windows are destroyed
        """
        if win in self.aux_win:
            self.aux_win.remove(win)

    def clean_subwin(self):
        """
        As it says
        """
        for win in self.aux_win:
            win.destroy()
        self.aux_win = []



#
## Filename    : text_page.py
## Author(s)   : Michel Le Borgne
## Created     : 10/2009
## Revision    :
## Source      :
##
## Copyright 2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided hereunder is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE
##
##
## Contributor(s): Geoffroy Andrieux, Nolwenn Le Meur
##
"""
Collection of widgets for text handling
"""
import gtksourceview2 as gtksourceview
import gtk
import gtk.glade

import pkg_resources

class TextEditConfig:
    """
    Text editors configuration
    """
    def __init__(self):
        self.style = "classical"
        self.show_line_numbers = False
        self.show_line_marks = True



class TextArea(gtksourceview.View):
    """
    GtkSourceView with supplementary attributes
    """
    def __init__(self, language = None):
        gtksourceview.View.__init__(self)
        # create buffer
        buffer = gtksourceview.Buffer()
        mgr = gtksourceview.style_scheme_manager_get_default()
        style_scheme = mgr.get_scheme('classical')
        if style_scheme:
            buffer.set_style_scheme(style_scheme)

        # set language
        if language:
            lma = gtksourceview.LanguageManager()
            spa = lma.get_search_path()
            spa.append("utils/language-specs")
            lma.set_search_path(spa)
            lang = lma.get_language(language)
            buffer.set_language(lang)

        # set buffer
        self.set_buffer(buffer)
        self.buffer = buffer

    def refresh_config(self, edit_config):
        """
        As it says
        """
        self.set_show_line_numbers(edit_config.show_line_numbers)
        self.set_show_line_marks(edit_config.show_line_marks)
        mgr = gtksourceview.style_scheme_manager_get_default()
        style_scheme = mgr.get_scheme(edit_config.style)
        #TODO
        if style_scheme:
            self.get_buffer().set_style_scheme(style_scheme)
        else:
            pass

class TextPage(gtk.Frame):
    """
    A simple text editor for pages
    """
    def __init__(self, language, edit_config = None):
        gtk.Frame.__init__(self)
        # text view and model
        self.write = TextArea(language)
        self.modified = False
        self.not_set = False
        # configuration
        if edit_config:
            self.write.refresh_config(edit_config)

        # page num in notebook (if used in notebook)
        self.page_num = -1

        # wrap in a scroll window
        swin = gtk.ScrolledWindow()
        swin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        swin.set_shadow_type(gtk.SHADOW_IN)
        swin.add_with_viewport(self.write)
        self.add(swin)

        # connect control
        self.write.buffer.connect("changed", self.is_modified)

    def refresh_config(self, edit_config):
        """
        Self explained
        """
        self.write.refresh_config(edit_config)

    def set_text(self, text):
        """
        Put text in the buffer
        """
        self.write.get_buffer().set_text(text)
        self.not_set = False
        self.modified = False

    def get_text(self):
        """
        Retrieve text from the buffer
        """
        wbuf = self.write.get_buffer()
        iter_begin = wbuf.get_start_iter()
        iter_end = wbuf.get_end_iter()
        text = wbuf.get_text(iter_begin, iter_end)
        return text

    def insert_at_end(self, text):
        """
        Insert text at the end of the buffer
        """
        wbuf = self.write.get_buffer()
        iter_end = wbuf.get_end_iter()
        wbuf.insert(iter_end, text)

    def clear(self):
        """
        RAZ of the buffer
        """
        wbuf = self.write.get_buffer()
        iter_start = wbuf.get_start_iter()
        iter_end = wbuf.get_end_iter()
        wbuf.delete(iter_start, iter_end)

    # control
    def is_modified(self, widget):
        """
        Mark as modified
        """
        self.modified = True
        #self.modified = self.not_set
        #self.not_set = True

class BioSignalEditor:
    """
    Biosignal simple editor - can load and save files
    @param on_leave: method to be used when quitting the editor
    """
    def __init__(self, name, master, on_leave):
        self.master = master
        self.what_todo = on_leave
        # gui
        # Set the Glade file
        template = pkg_resources.resource_filename(
            __name__,
            "../chart_glade/biosiged.glade"
        )
        self.wtree = gtk.glade.XML(template)

        # Get the Main Window, and connect the "destroy" event
        self.main_window = self.wtree.get_widget("window1")
        if (self.main_window):
            self.main_window.connect("delete_event", self.on_destroy)
        self.main_window.set_title(name)
        self.main_window.set_default_size(800, 900)
        self.main_window.set_position(gtk.WIN_POS_CENTER)
        # text_area
        econf = TextEditConfig();
        self.page = TextPage("biosignal", econf)
        vbox =  self.wtree.get_widget("vbox1")
        vbox.pack_start(self.page)

        # file menu
        menu_item =  self.wtree.get_widget("new_m")
        menu_item.connect("activate", self.on_new)
        menu_item =  self.wtree.get_widget("open_m")
        menu_item.connect("activate", self.on_open)
        menu_item =  self.wtree.get_widget("save_m")
        menu_item.connect("activate", self.on_save)
        menu_item =  self.wtree.get_widget("save_as_m")
        menu_item.connect("activate", self.on_save_as)
        menu_item =  self.wtree.get_widget("quit_m")
        menu_item.connect("activate", self.on_destroy)

        # style menu
        #TODO

        # internals
        self.file = None

        self.main_window.show_all()

    def on_new(self, widget):
        """
        Clear callback
        """
        self.page.clear()
        pass

    def choose_file(self):
        """
        As it says
        """
        choice = gtk.FileChooserDialog("Biosignal Editor", None ,
                                       gtk.FILE_CHOOSER_ACTION_OPEN,
                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                        gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        choice.set_default_response(gtk.RESPONSE_OK)

        #add a filter to see only biosig files
        filter = gtk.FileFilter()
        filter.set_name("bsi files")
        filter.add_pattern("*.bsi")
        choice.add_filter(filter)

        #add a filter to see all
        no_filter = gtk.FileFilter()
        no_filter.set_name("all")
        no_filter.add_pattern("*")
        choice.add_filter(no_filter)

        response = choice.run()
        if response == gtk.RESPONSE_OK:
            fname =  choice.get_filename()
        elif response == gtk.RESPONSE_CANCEL:
            fname =  None
        choice.destroy()
        return fname

    def on_open(self, widget):
        """
        open a window to search .bsi file
        """
        file_name = self.choose_file()
        if file_name:
            file = open(file_name, 'r+')
            self.file = file
            text = file.read()
            self.page.set_text(text)

    def on_save(self, widget):
        """
        Save callback
        """
        if not self.file:
            file_name = self.choose_file()
            if file_name:
                self.file = open(file_name, 'r+')
            else:
                return
        text = self.page.get_text()
        self.file.write(text)

    def on_save_as(self, widget):
        """
        Save as callback
        """
        file_name = self.choose_file()
        if file_name:
            file = open(file_name, 'r+')
            self.file = file
            text = self.page.get_text()
            print 'SAVE AS', text
            self.file.write(text)

    def on_destroy(self, widget, xxx=None):
        """
        destroy callback
        """
        self.what_todo(self)
        self.main_window.destroy()

    def set_text(self, text):
        """
        As it says
        """
        self.page.set_text(text)

    def get_text(self):
        """
        As it says
        """
        return self.page.get_text()



# -*- coding: utf-8 -*-
# Copyright (C) 2010-2017  IRISA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# The original code contained here was initially developed by:
#
#     Geoffroy Andrieux.
#     IRISA/IRSET
#     Symbiose team
#     IRISA  Campus de Beaulieu
#     35042 RENNES Cedex, FRANCE
#
#
# Contributor(s): Michel Le Borgne, Nolwenn Le Meur, Pierre Vignet
"""
Created on Nov 2016

..sectionauthor:: Pierre Vignet <pierre.vignet@irisa.fr>
"""

def main():

    """Launch Cadbiom gui"""
    import gtk
    from cadbiom_gui.gt_gui.charter import Charter

    charter = Charter(None)
    charter.show()
    gtk.main()



#
## Filename    : warn.py
## Author(s)   : Michel Le Borgne
## Created     : 11/2009
## Revision    : 
## Source      : 
##
## Copyright 2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided hereunder is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE 
##     
##
## Contributor(s): Geoffroy Andrieux, Nolwenn Le Meur
##

"""
 A collection of warning widgets
"""

import pygtk
pygtk.require('2.0')
import gtk

def start_warn(message):
    """
    Warn for a start
    """
    mess = gtk.MessageDialog(None,
                            gtk.DIALOG_MODAL,
                            gtk.MESSAGE_INFO,
                            gtk.BUTTONS_OK,
                            message)
    mess.run()
    mess.destroy()

# OK
def ok_warn(message, color =None):
    """
    Simple warning before action
    """
    mess = gtk.MessageDialog(None,
                            gtk.DIALOG_MODAL,
                            gtk.MESSAGE_INFO,
                            gtk.BUTTONS_OK,
                            message)
    if color:
        color = gtk.gdk.color_parse(color)
        mess.modify_bg(gtk.STATE_NORMAL, color)
    mess.run()
    mess.destroy()

# error - imply cancellation of the action
def cancel_warn(message):
    """
    error - imply cancellation of the action
    """
    message1 = message+" - CANCELLED"
    mess = gtk.MessageDialog(None,
                            gtk.DIALOG_MODAL,
                            gtk.MESSAGE_WARNING,
                            gtk.BUTTONS_OK,
                            message1)
    mess.run()
    mess.destroy()

# fatal error - bug or bd problems
def system_warn(message):
    """
    fatal error - bug or bd problems
    """
    message1 = message+" - BD problem or BUG!"
    mess = gtk.MessageDialog(None,
                            gtk.DIALOG_MODAL,
                            gtk.MESSAGE_ERROR,
                            gtk.BUTTONS_OK,
                            message1)
    mess.run()
    mess.destroy()
# ask for action
def confirm(parent, message):
    """
    ask for action
    """
    dialog = gtk.Dialog("Warning",
                        parent,
                        gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                        (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                         gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
    label = gtk.Label(message)
    dialog.vbox.pack_start(label, True, False, 0)
    label.show()
    response = dialog.run()
    dialog.destroy()
    if response == gtk.RESPONSE_ACCEPT:
        return True
    elif  response == gtk.RESPONSE_REJECT:
        return False

# util functions for buttons with image
def xpm_label_box(parent, xpm_filename, label_text):
    """
    Create box for xpm and label
    """
    box1 = gtk.HBox(False, 0)
    box1.set_border_width(2)
    # Now on to the image stuff
    image = gtk.Image()
    image.set_from_file(xpm_filename)

    # Create a label for the button
    label = gtk.Label(label_text)

    # Pack the pixmap and label into the box
    box1.pack_start(image, False, False, 3)
    box1.pack_start(label, False, False, 3)
   
    image.show()
    label.show()
    return box1

# utils for DB
# wrap class for retrieving values from gui
class Wrap(object):
    """
    Hack
    """
    def __init__(self):
        self.val = None
        

# dialog entry
class DialogEntry(gtk.Dialog):
    """
    Custom dialog entry
    """
    
    def __init__(self, mess):
        gtk.Dialog.__init__(self, mess,
                            None,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
        
        self.set_default_size(350, 75)
        self.entry = gtk.Entry()
        self.vbox.pack_start(self.entry, True, True, 0)
        self.entry.show()

        self.okb = gtk.Button("Ok")
        self.action_area.pack_start(self.okb, False, False, 0)
        self.okb.show()

        self.cancel = gtk.Button("Cancel")
        self.action_area.pack_start(self.cancel, False, False, 0)
        self.cancel.show()





##
## Filename    : notebookUtils.py
## Author(s)   : Michel Le Borgne
## Created     : 4/2010
## Revision    : 
## Source      : 
##
## Copyright 2009-2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided hereunder is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE 
##     
##
## Contributor(s): Geoffroy Andrieux, Nolwenn Le Meur
##

"""
Utilities for notebooks
"""
import gtk

def add_icon_to_button(button):
    """
    add a close buttpn to tab
    """
    #hbox
    iconBox = gtk.HBox(False, 0)
    #empty image
    image = gtk.Image()
    #get close button icon
    image.set_from_stock(gtk.STOCK_CLOSE,gtk.ICON_SIZE_MENU)
    #remove relief
    gtk.Button.set_relief(button,gtk.RELIEF_NONE)
    #get button properties
    settings = gtk.Widget.get_settings(button)
    # w and h = dimensions
    (w,h) = gtk.icon_size_lookup_for_settings(settings,gtk.ICON_SIZE_MENU)
    #we modify dimensions
    gtk.Widget.set_size_request(button, w + 4, h + 4)
    image.show()
    #put the image in the box
    iconBox.pack_start(image, True, False, 0)
    #put the box in the button
    button.add(iconBox)
    iconBox.show()
    return 

def create_custom_tab(text):
    """
    Customized tab with label and close button
    """
    #event box creation
    eventBox = gtk.EventBox()
    #hbox
    tab_box = gtk.HBox(False, 2)
    #label "text"
    tab_label = gtk.Label(text)
    #buton
    tab_button=gtk.Button()

    #add the close icon to button
    add_icon_to_button(tab_button)

    eventBox.show()
    tab_button.show()
    tab_label.show()
    #connect label and button to the box
    tab_box.pack_start(tab_label, False)
    tab_box.pack_start(tab_button, False)

    tab_box.show_all()
    #add the box to eventbox
    eventBox.add(tab_box)
    return (eventBox, tab_button)




##
## Filename    : displayFact.py
## Author(s)   : Michel Le Borgne
## Created     : 11/2009
## Revision    : 
## Source      : 
##
## Copyright 2009-2010 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided hereunder is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE 
##     
##
## Contributor(s): Geoffroy Andrieux, Nolwenn Le Meur
##

"""
Widget for lists used in Cadbiom Gui
"""
import gtk
import gobject

class ToggleList (gtk.Frame):
    """
    List view with toggle button
    """
    def __init__(self): 
        """
        Create a view 
        """
        gtk.Frame.__init__(self)
        self.view = gtk.TreeView()
        self.view.show()
        self.add(self.view)
        self.height = 0
        
        # model
        self.list_item = gtk.ListStore(gobject.TYPE_STRING, 
                                       gobject.TYPE_BOOLEAN)
        self.view.set_model(self.list_item)
        
        # CellRendererText for names
        self.renderer0 = gtk.CellRendererText()
        self.renderer0.set_property('editable', False)

        # CellRendererToggle for selection
        self.renderer1 = gtk.CellRendererToggle()
        self.renderer1.set_property('activatable', True)
        self.renderer1.connect('toggled', self.toggled_col1_callback, 
                               self.list_item)
        # connection of cell renderers to respective columns
        self.column0 = gtk.TreeViewColumn(None, self.renderer1)
        self.column1 = gtk.TreeViewColumn(None, self.renderer0, text=0 )
        self.column0.add_attribute(self.renderer1, "active", 1)

        self.column0.set_property("sizing", gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.column1.set_property("sizing", gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.view.append_column(self.column0 )
        self.view.append_column(self.column1 )
        
    def toggled_col1_callback( self, cellrenderer, path, model):
        """
        callback for column1
        """
        model[path][1] = not model[path][1]
        return

    def set_list_select(self, l_keywords):
        """
        initialize from a list of name - all unselected
        @param l_keywords: list<string> 
        """
        height = 0
        for kwo in l_keywords:
            height = height + 40
            self.list_item.append([kwo, False])
        self.height = height
        
    def set_list_pairs(self, l_pairs):
        """
        initialize from a list of pairs - all unselected
        @param l_pairs: list< pair<string,bool> > 
        """
        height = 0
        for pair in l_pairs:
            height = height + 40
            self.list_item.append([pair[0], pair[1]])
        self.height = height
        
    def get_selected_items(self):
        """
        retrieve the selected items
        """
        l_kw = []
        for item in self.list_item:
            if item[1]:
                l_kw.append(item[0])
        return l_kw
    
    def set_selected_items(self, selected_list):
        """
        Aqs it says
        """
        for item in self.list_item:
            if item[0] in selected_list:
                item[1] = True   
            else :
                item[1] = False        

    def get_pair_values(self):
        """
        retrieve the cas and word associated
        """
        l_pairs = []
        for item in self.list_item:
                l_pairs.append((item[0], item[1]))
        return l_pairs

    def clear(self, widget):
        """
        it is clear
        """
        for item in self.list_item:
            item[1] = False
            
    def select_all(self, widget):
        """
        self explained
        """
        for item in self.list_item:
            item[1] = True
            
    def rubout(self):
        """
        remove all items
        """
        self.list_item.clear()

    def refresh(self, newlist):
        """
        self explained
        """
        self.list_item.clear()
        self.set_list_select(newlist)

class ChoiceList(object):
    """
    Display a list with marks
    """   
    def __init__(self, l_names, callback, title):
        self.callback = callback
        
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_resizable(True)
        self.window.connect("destroy", gtk.main_quit)
        height = gtk.gdk.screen_height()
        height = int(height * 0.85)
        self.window.set_size_request(300, height)
        self.window.set_title(title)
 
        # view
        self.choice_view = gtk.TreeView()
        
        # model
        self.choice_list = gtk.ListStore(gobject.TYPE_STRING)
        l_names.sort()
        for name in l_names:
            self.choice_list.append([name])
        self.choice_view.set_model(self.choice_list)
        
        # CellRendererText for choice names
        self.renderer0 = gtk.CellRendererText()
        self.renderer0.set_property('editable', False)

        # connection of cell renderers to respective columns
        self.column0 = gtk.TreeViewColumn("", self.renderer0, text=0)
        self.choice_view.append_column(self.column0 )

        swin = gtk.ScrolledWindow()
        swin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        swin.set_shadow_type(gtk.SHADOW_IN)
        swin.add_with_viewport(self.choice_view)
        self.window.add(swin)
        
        # control
        self.choice_view.connect("row_activated", 
                                 self.choice_selection_callback)
        self.window.show_all()

    def choice_selection_callback(self, widget, path, view_column):
        """
        callback called when a choice is done
        """
        if widget == self.choice_view:
            treeselection = widget.get_selection()
            (choice, iter) = treeselection.get_selected()
            choice_name = choice.get_value(iter, 0)
            self.callback(choice_name)
            self.window.destroy()
        
        


## Filename    : charter_info.py
## Author(s)   : Geoffroy Andrieux
## Created     : 03/2010
## Revision    :
## Source      :
##
## Copyright 2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided here under is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Geoffroy Andrieux.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE
##
##
## Contributor(s): Michel Le Borgne, Nolwenn Le Meur
##
"""
Widget for displaying informations on the elements of the model
"""
import gtk

from utils.text_page import TextEditConfig, TextPage

import pkg_resources

class CharterInfo(gtk.Frame):
    """
    windows of charter_info
    """
    def __init__(self, frame):
        self.frame = frame
        self.controler = None
        self.fact_window = None
        self.model_info = ModelInfo(self)
        self.node_info = NodeInfo(self)
        self.trans_info = TransInfo(self)

        self.current_info = self.model_info

    def set_controler(self, new_controler):
        """
        Associate a controler in a MVC pattern
        """
        if self.controler:
            self.controler.detach("current_change", self)
        self.controler = new_controler
        self.controler.attach("current_change", self)

    def display(self):
        """
        Show all
        """
        chi = self.frame.get_child()
        if chi:
            self.frame.remove(chi)

        self.frame.add(self.current_info.frame)
        self.frame.show()


    def switch_to(self, new_info):
        """
        callback when info change
        """
        if self.current_info == None:
            self.current_info = new_info
            self.display()
        else:
            self.current_info.warn_change()
            self.current_info = new_info
            self.display()


    def update(self, node, transition):
        """
        Used if registered as an observer
        """
        if (node == None and transition == None):
            self.model_info.update(self.controler)
            self.switch_to(self.model_info)

        elif node != None and transition == None:
            if node.is_top_node():
                self.model_info.update(self.controler)
                self.switch_to(self.model_info)
            else:
                self.node_info.update(node)
                self.switch_to(self.node_info)

        else :
            self.trans_info.update(transition)
            self.switch_to(self.trans_info)



    def enter_callback(self, widget, entry):
        """
        return the entry
        """
        entry_text = entry.get_text()
        print "Entry contents: %s\n" % entry_text

    def has_transition(self):
        """
        As it says
        """
        return self.current_info.has_transition()


class ModelInfo(object):
    """
    Object used as a model - store informations
    """
    def __init__(self, cin):
        template = pkg_resources.resource_filename(
            __name__,
            "chart_glade/model_info.glade"
        )
        self.page = gtk.glade.XML(template)
        wid = self.page.get_widget("model_frame")
        self.frame = wid.get_child()
        wid.remove(self.frame)
        self.info = cin

        self.name = self.page.get_widget("model_entry")
        self.name.connect("changed", self.set_model)
        self.name.set_editable(True)

        self.nb_nodes = self.page.get_widget("node_entry")
        self.nb_nodes.set_editable(False)
        self.nb_nodes.set_sensitive(False)

        self.nb_trans = self.page.get_widget("trans_entry")
        self.nb_trans.set_editable(False)
        self.nb_trans.set_sensitive(False)

    def set_model(self, widget):
        """
        As it says
        """
        self.model.name = self.name.get_text()
        self.model.notify()

    def warn_change(self):
        """
        As it says
        """
        pass

    def update(self, controler):
        """
        when used as an observer
        """
        self.model = controler.model
        self.name.set_text(controler.model.name)
        nodes = str(len(self.model.simple_node_dict.keys()))
        trans = str(len(self.model.transition_list))
        self.nb_nodes.set_text(nodes)
        self.nb_trans.set_text(trans)

    def has_transition(self):
        """
        As it says
        """
        return None



class NodeInfo(object):
    """
    A componant of ModelInfo used for nodes
    """
    def __init__(self, cin):
        template = pkg_resources.resource_filename(
            __name__,
            "chart_glade/node_info.glade"
        )
        self.page = gtk.glade.XML(template)
        wid = self.page.get_widget("node_frame")
        self.frame = wid.get_child()
        wid.remove(self.frame)
        self.info = cin

        self.name = self.page.get_widget("node_entry")
        self.name.connect("changed", self.set_node)

    def set_node(self, widget):
        """
        As it says
        """
        self.node.set_name(self.name.get_text())
        self.node.model.notify()

    def warn_change(self):
        """
        As it says
        """
        pass

    def update(self, node):
        """
        Used when registered as an observer
        """
        if node:
            self.node = node
            self.name.set_text(node.name)

    def has_transition(self):
        """
        As it says
        """
        return None

class TransInfo(object):
    """
    A component of ModelInfo used for transitions
    """
    def __init__(self, cin):

        template = pkg_resources.resource_filename(
            __name__,
            "chart_glade/trans_info.glade"
        )
        self.page = gtk.glade.XML(template)
        wid = self.page.get_widget("trans_frame")
        self.frame = wid.get_child()
        wid.remove(self.frame)
        self.info = cin # owner
        self.note_window = None

        self.name = self.page.get_widget("trans_entry")
        self.name.set_editable(False)
        self.evt = self.page.get_widget("evt_entry")
        self.cond = self.page.get_widget("cond_entry")
        self.noteb = self.page.get_widget("note")
        self.show = self.page.get_widget("butshow")

        self.name.connect("changed", self.set_trans)
        self.evt.connect("changed", self.set_trans)
        self.cond.connect("changed", self.set_trans)
        self.noteb.connect("clicked", self.set_note)

    def set_trans(self, widget):
        """
        As it says
        """
        if self.lock:
            return
        else:
            self.trans.set_name(self.name.get_text())
            self.trans.set_event(self.evt.get_text())
            self.trans.set_condition(self.cond.get_text())
            self.trans.ori.model.notify()

    def warn_change(self):
        """
        As it says
        """
        self.save_note(None)

    def update(self, trans):
        """
        When registered as an observer
        """
        if trans:
            self.lock = True
            self.trans = trans
            self.name.set_text(trans.ori.name+'->'+trans.ext.name)
            self.evt.set_text(trans.event)
            self.cond.set_text(trans.condition)
            self.trans = trans # for fact association management
            self.lock = False

    def has_transition(self):
        """
        As it says
        """
        return self.trans

    def set_note(self, widget):
        """
        Associate a text note to a transition
        """
        trans = self.trans
        self.note_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.note_window.connect("destroy", self.save_note)
        self.note_window.set_title("notes: "+trans.ori.name+"->"+trans.ext.name)
        self.note_window.set_position(gtk.WIN_POS_CENTER)
        self.note_window.set_keep_above(True)
        self.note_window.set_default_size(600, 800)
        ed_conf = TextEditConfig()
        self.note_text_page = TextPage(None, ed_conf)
        self.note_text_page.set_text(trans.note)
        self.note_window.add(self.note_text_page)
        self.note_window.show_all()
        return

    def save_note(self, widget):
        """
        register the note
        """
        if self.note_window:
            text = self.note_text_page.get_text()
            self.trans.note = text
            self.trans.ori.model.modified = True
            self.note_window.destroy()
            self.note_window = None



## Filename    : chart_stat_controler.py
## Author(s)   : Michel Le Borgne
## Created     : 08/2012
## Revision    :
## Source      :
##
## Copyright 2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License  published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided here under is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE
##
##
## Contributor(s): Geoffroy Andrieux
##
"""
Collection of widgets and controlers for static analysis
"""
import gtk
from cadbiom.models.guard_transitions.analyser.static_analysis import StaticAnalyzer
from cadbiom_gui.gt_gui.utils.reporter import CompilReporter

import pkg_resources

import sys
sys.setrecursionlimit(10000)

class ChartStatControler(object):
    """
    Control of static analysis

    /!\ This class is fully deprecated and is no longer used /!\
    """
    def __init__(self, emvc, reporter):
        self.edit_mvc = emvc
        self.reporter = reporter                   # display error messages
        # graphical interface
        template = pkg_resources.resource_filename(
            __name__,
            "../chart_glade/static_analysis.glade"
        )
        self.wtree = gtk.glade.XML(template)
        self.main_window = self.wtree.get_widget("window1")
        title = "Static analysis - " + self.edit_mvc.model.name
        self.main_window.set_title(title)
        if (self.main_window):
            self.main_window.connect("destroy", self.on_destroy)
        self.main_window.set_position(gtk.WIN_POS_CENTER)
        #self.main_window.set_keep_above(True)
        self.main_window.resize(150, 300)

        # completion button
#        button = self.wtree.get_widget("completionbutton")
#        button.connect("clicked", self.on_completion)

        # frontier button
        button = self.wtree.get_widget("frontierbutton")
        button.connect("clicked", self.on_frontier)

        #  basal activated genes button
        button = self.wtree.get_widget("bgenebutton")
        button.connect("clicked", self.on_b_gene)

        # dependance graph button
        button = self.wtree.get_widget("depgraphbutton")
        button.connect("clicked", self.on_depgraph)

        # stat button
        button = self.wtree.get_widget("statbutton")
        button.connect("clicked", self.on_stat)

        # register as auxiliary window
        self.edit_mvc.win_register(self)
        self.subwin = []

        # display
        self.main_window.show_all()


#    def on_completion(self, widget):
#        """
#        TODO
#        """
#        pass

    def on_frontier(self, widget):
        """
        Compute connected components which are on the frontier
        """
        reporter = CompilReporter()
        stan = StaticAnalyzer(reporter)
        stan.build_from_chart_model(self.edit_mvc.model)
        # errors??
        lscc = stan.get_frontier_scc()
        display_w =  SCCWindow(lscc, self.edit_mvc, self.reporter, self)


    def on_b_gene(self, widget):
        """
        Compute basal activated genes
        """
        reporter = CompilReporter()
        stan = StaticAnalyzer(reporter)
        stan.build_from_chart_model(self.edit_mvc.model)
        lwbag = stan.get_why_basal_genes()
        display_w =  BAGWindow(lwbag, self.edit_mvc, self.reporter, self)

    def on_depgraph(self, widget):
        """
        Computation and export of the dependance graph
        """
        DependencyGraphWindow(self.edit_mvc, self.reporter, self)

    def on_stat(self, widget):
        """
        Fill window with model informations.
        """
        reporter = CompilReporter()
        # get stats from StaticAnalyzer
        stan = StaticAnalyzer(reporter)
        stan.build_from_chart_model(self.edit_mvc.model)
        ststat = stan.get_statistics()
        display_w = STATWindow(ststat, self.edit_mvc, self.reporter, self)

    def win_register(self, win):
        """
        register a sub wundow
        """
        print "register"
        self.subwin.append(win)

    def win_remove(self, win):
        """
        unregister a sub window
        """
        print "remove"
        self.subwin.remove(win)

    def clean_subwin(self):
        """
        sub windows management
        """
        for win in self.subwin:
            print "clean"
            win.destroy()
        self.subwin = []

    def on_destroy(self, widget):
        """
        Standard call back
        """
        print "on destroy"
        self.clean_subwin()
        if self.main_window:
            self.main_window.destroy()

    def destroy(self):
        """
        Standard call back
        """
        self.on_destroy(None)


class  DependencyGraphWindow(object):
    """
    Dialog for dependency graph analysis and export
    /!\ This class is fully deprecated and is no longer used /!\
    """
    def __init__(self, emvc, reporter, parent):
        self.emvc = emvc
        self.reporter = reporter
        self.parent = parent

        # window creation
        template = pkg_resources.resource_filename(
            __name__,
            "../chart_glade/dependency_graph.glade"
        )
        self.wtree = gtk.glade.XML(template)
        self.main_window = self.wtree.get_widget("window1")
        self.main_window.set_title("Dependency graph - "+self.emvc.model.name)

        self.main_window.set_resizable(True)
        screen_h = gtk.gdk.screen_height()
        screen_h = int(screen_h * 0.20)
        self.main_window.set_size_request(300, screen_h)

        if (self.main_window):
            self.main_window.connect("destroy", self.on_destroy)

        self.main_window.set_position(gtk.WIN_POS_CENTER)

        # transition graph radio button
        rbutton = self.wtree.get_widget("transGraph_rb")
        rb_name = rbutton.get_name()
        rbutton.connect("toggled", self.rb_callback, rb_name)

        # dependency graph radio button
        rbutton = self.wtree.get_widget("dpGraph_rb")
        rb_name = rbutton.get_name()
        rbutton.connect("toggled", self.rb_callback, rb_name)

        # full dependency graph radio button
        rbutton = self.wtree.get_widget("fullDpGraph_rb")
        rb_name = rbutton.get_name()
        rbutton.connect("toggled", self.rb_callback, rb_name)

        # DOT button
        button = self.wtree.get_widget("dotButton")
        button.connect("clicked", self.on_save_dot)

        # graphml button
        button = self.wtree.get_widget("graphmlButton")
        button.connect("clicked", self.on_save_graphml)

        # CREATE STATIC ANALYZER AND LOAD MODEL
        self.stan = StaticAnalyzer(reporter)
        self.stan.build_from_chart_model(self.emvc.model)

        self.graph_type = "transGraph_rb"

        # register as auxiliary window
        self.parent.win_register(self.main_window)

        # display
        self.main_window.show_all()

    def rb_callback(self, widget, name):
        """
        ???
        """
        self.graph_type = name

    def on_save_dot(self, widget):
        """
        compute a dependency graph and open a window to save as dot file
        """
        if self.graph_type == "transGraph_rb":
            graph = self.stan.make_transition_dg()
        elif self.graph_type == "dpGraph_rb":
            graph = self.stan.make_dependence_dg(True)
        else :
            graph = self.stan.make_full_dependence_dg(True)
        ch_opt = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE,
                   gtk.RESPONSE_OK)
        choice = gtk.FileChooserDialog("Save as dot file",
                                       None,
                                       gtk.FILE_CHOOSER_ACTION_SAVE, ch_opt)
        choice.set_default_response(gtk.RESPONSE_OK)

        #add a filter to see only dot files (*.dot)
        gfilter = gtk.FileFilter()
        gfilter.set_name("dot files")
        gfilter.add_pattern("*.dot")
        choice.add_filter(gfilter)

        #add a filter to see all
        no_filter = gtk.FileFilter()
        no_filter.set_name("all")
        no_filter.add_pattern("*")
        choice.add_filter(no_filter)

        response = choice.run()
        if response == gtk.RESPONSE_OK:
            self.stan.export_2_dot(graph, choice.get_filename())
        elif response == gtk.RESPONSE_CANCEL:
            pass
            #print 'Closed, no files selected'
        choice.destroy()

    def on_save_graphml(self, widget):
        """
        compute a dependency graph and open a window to save as graphml file
        """
        if self.graph_type == "transGraph_rb":
            graph = self.stan.make_transition_dg()
        elif self.graph_type == "dpGraph_rb":
            graph = self.stan.make_dependence_dg(True)
        else :
            graph = self.stan.make_full_dependence_dg(True)

        ch_opt = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE,
                   gtk.RESPONSE_OK)
        choice = gtk.FileChooserDialog("Save as graphml file", None,
                                       gtk.FILE_CHOOSER_ACTION_SAVE, ch_opt)
        choice.set_default_response(gtk.RESPONSE_OK)

        #add a filter to see only graphml files (*.graphml)
        filter = gtk.FileFilter()
        filter.set_name("graphml files")
        filter.add_pattern("*.graphml")
        choice.add_filter(filter)

        #add a filter to see all
        no_filter = gtk.FileFilter()
        no_filter.set_name("all")
        no_filter.add_pattern("*")
        choice.add_filter(no_filter)

        response = choice.run()
        if response == gtk.RESPONSE_OK:
            self.stan.export_2_graphml(graph, choice.get_filename())
        elif response == gtk.RESPONSE_CANCEL:
            pass
            #print 'Closed, no files selected'
        choice.destroy()

    def on_destroy(self, widget):
        """
        Standard call back
        """
        if self.main_window:
            self.parent.win_remove(self.main_window)
            self.main_window.destroy()

    def destroy(self):
        """
        For linked windows
        """
        self.on_destroy(None)



class SCCWindow(object):
    """
    Class used for displaying frontier isolated components
    """
    def __init__(self, l_fsol, emvc, reporter, parent):
        """
        @param l_fsol: list<list<string>>
        @param emvc: edit_mvc
        @param reporter: Classical reporter
        @param parent: parent widget
        """

        self.emvc = emvc
        self.reporter = reporter

        self.compt = 0
        self.choice = 0
        self.textview_list = []
        self.l_solutions = l_fsol
        self.aux_win = [] # for clean_subwin works properly

        # window creation
        template = pkg_resources.resource_filename(
            __name__,
            "../chart_glade/checker_solutions.glade"
        )
        self.wtree = gtk.glade.XML(template)
        self.window = self.wtree.get_widget("window1")

        self.window.set_resizable(True)
        self.window.set_title("Isolated Strongly Connected Components (SCC) - "\
                              + self.emvc.model.name)
        height = gtk.gdk.screen_height()
        height = int(height * 0.30)
        self.window.set_size_request(700, height)

        # Set window above all windows
        self.window.set_transient_for(parent.main_window)

        # Event on_escape key pressed
        self.window.connect('key_press_event', self.on_escape)

        # button save choice
        button = self.wtree.get_widget("but_sv_choice")
        button.connect("clicked", self.on_save, False)

        # button save all
        button = self.wtree.get_widget("but_sv_all")
        button.connect("clicked", self.on_save, True)
        if len(self.l_solutions) == 1:
            button.set_sensitive(False)

        # display
        self.display_info()

        # remove bottom buttons
        hbbox =  self.wtree.get_widget("hbuttonbox2")
        vbox = self.wtree.get_widget("vbox1")
        vbox.remove(hbbox)

        # register
        parent.win_register(self.window)
        self.parent = parent

        # display
        self.window.show_all()
        #gtk.main()

    def on_destroy(self, widget):
        """
        when leaving the window
        useless ? no sub window is created from this mother class and from
        subclasses...
        """
        self.clean_subwin()
        self.parent.win_remove(self)
        if self.window:
            self.window.destroy()

    def destroy(self):
        """
        when leaving the parent window
        """
        self.on_destroy(None)

    def on_escape(self, widget, event):
        """On ESC key_press_event, destroy this window."""
        if gtk.gdk.keyval_name(event.keyval) == "Escape":
            self.window.destroy()


    def display_info(self):
        """
        Display info on isolated SCC
        """
        SCC_found = (len(self.l_solutions[0]) != 0)

        # info label
        info = self.wtree.get_widget("label_nbsol")
        if not SCC_found:
            info.set_text("No isolated Strongly Connected Components found.")
            return
        else:
            info.set_text(str(len(self.l_solutions)) + \
                          " isolated Strongly Connected Components found.\n"
                          "(Only 1 node of each of them requires an input "
                          "transition with a StartNode)")

        self.frame_sol =  self.wtree.get_widget("frame_sol")
        # button frame
        vbox = gtk.VBox(False, 0)
        self.frame_sol.add(vbox)
        vbox.show()

        scroll = gtk.ScrolledWindow()
        vbox.pack_start(scroll)

        vbox2 = gtk.VBox()
        scroll.add_with_viewport(vbox2)
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.button_group = None

        frame = self.build_button_frame(self.l_solutions[0])
        vbox2.pack_start(frame)

        for sol in self.l_solutions[1:]:
            hsep = gtk.HSeparator()
            vbox2.pack_start(hsep, False, False)

            frame = self.build_button_frame(sol, self.button_group)
            vbox2.pack_start(frame)


    def build_button_frame(self, state_list, group = None):
        """
        Display info on each isolated SCC
        """
        template = pkg_resources.resource_filename(
            __name__,
            "../chart_glade/button_frame.glade"
        )
        wtree = gtk.glade.XML(template)
        wid = wtree.get_widget("button_frame")
        frame = wid.get_child()
        wid.remove(frame)

        #textview
        tname = 'tw'+str(self.compt)
        text = wtree.get_widget("textview")
        text.set_name(tname)

        text_buffer = text.get_buffer()
        str_state = '\n'
        for state in state_list :
            str_state += state + '\t'
        str_state = str_state + '\n'
        text_buffer.set_text(str_state)
        text.set_editable(False)

        self.textview_list.append(state_list)

        #radio button
        rname = 'rb' + str(self.compt)
        rbut = wtree.get_widget("rButton")
        rbut.set_name(rname)

        if group:
            rbut.set_group(group)
        else :
            self.button_group = rbut
            rbut.set_active(True)

        rbut.connect("toggled", self.rb_callback, self.compt)
        self.compt += 1
        return frame

    def rb_callback(self, widget, id_frame):
        """
        radio button call_back
        """
        self.choice = id_frame

    def on_save(self, widget, all_sol=True):
        """
        open a window to save as xml file
        @param all_sol : boolean - True if we take all solutions
        """
        choice = gtk.FileChooserDialog("Save solution", None ,
                                       gtk.FILE_CHOOSER_ACTION_SAVE,
                                       ( gtk.STOCK_CANCEL,
                                         gtk.RESPONSE_CANCEL,
                                         gtk.STOCK_SAVE,
                                         gtk.RESPONSE_OK))
        choice.set_default_response(gtk.RESPONSE_OK)

        #add a filter to see only xml files for biochart (*.txt)
        ffilter = gtk.FileFilter()
        ffilter.set_name("txt files")
        ffilter.add_pattern("*.txt")
        choice.add_filter(ffilter)

        #add a filter to see all
        no_filter = gtk.FileFilter()
        no_filter.set_name("all")
        no_filter.add_pattern("*")
        choice.add_filter(no_filter)

        response = choice.run()
        if response == gtk.RESPONSE_OK:
            self.create_solution_file(choice.get_filename(), all_sol)
        elif response == gtk.RESPONSE_CANCEL:
            pass
            #print 'Closed, no files selected'
        choice.destroy()

    def create_solution_file(self, sol_file, all_bool):
        """
        Print solutions in a file
        """
        if all_bool:
            scc_places = self.l_solutions
        else :
            scc_places = [self.l_solutions[self.choice]]

        sfile = open(sol_file,'w')
        for i in range(len(scc_places)):
            spla = scc_places[i]
            for elemt in spla :
                sfile.write(str(elemt)+'\t')

            sfile.write('\n-------------------------------------------\n\n\n')
        sfile.close()

    # sub windows management
    def win_register(self, win):
        """
        sub windows management
        """
        print "register subsubwin"
        if win.__class__ == ChartSimulControler:
            self.simul = win
        self.aux_win.append(win)

    def win_remove(self, win):
        """
        sub windows management
        """
        if win in self.aux_win:
            self.aux_win.remove(win)
        if win == self.simul:
            self.simul = None

    def clean_subwin(self):
        """
        sub windows management
        """
        for win in self.aux_win:
            win.destroy()
        self.aux_win = []


class BAGWindow(SCCWindow):
    def __init__(self, l_fsol, emvc, reporter, parent):
        """
        @param l_fsol: list<list<string>>
        @param emvc: edit_mvc
        @param reporter: Classical reporter
        @param parent: parent widget
        """

        SCCWindow.__init__(self, l_fsol, emvc, reporter, parent)
        self.window.set_title("Basal activated genes - "+self.emvc.model.name)

    def display_info(self):
        """
        ???
        """

        SCC_found = (len(self.l_solutions[0]) != 0)

        # info label
        info = self.wtree.get_widget("label_nbsol")
        if not SCC_found:
            info.set_text("No basal activated genes found.")
            return
        else:
            info.set_text(str(len(self.l_solutions)) + \
                          " basal activated genes found.")

        self.frame_sol =  self.wtree.get_widget("frame_sol")
        # button frame
        vbox = gtk.VBox(False, 0)
        self.frame_sol.add(vbox)
        vbox.show()

        scroll = gtk.ScrolledWindow()
        vbox.pack_start(scroll)

        vbox2 = gtk.VBox()
        scroll.add_with_viewport(vbox2)
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.button_group = None

        frame = self.build_button_frame(self.l_solutions[0])
        vbox2.pack_start(frame)

        for sol in self.l_solutions[1:]:
            hsep = gtk.HSeparator()
            vbox2.pack_start(hsep, False, False)

            frame = self.build_button_frame(sol, self.button_group)
            vbox2.pack_start(frame)


    def build_button_frame(self, pair, group = None):
        """
        ???
        """
        template = pkg_resources.resource_filename(
            __name__,
            "../chart_glade/button_frame.glade"
        )
        wtree = gtk.glade.XML(template)
        wid = wtree.get_widget("button_frame")
        frame = wid.get_child()
        wid.remove(frame)

        #textview
        tname = 'tw'+str(self.compt)
        text = wtree.get_widget("textview")
        text.set_name(tname)

        text_buffer = text.get_buffer()
        mess = '\nGene:'
        mess = mess + '\t' + pair[0]
        why = pair[1]
        mess = mess + "\n" + "Essential Inhib.: "
        for act in why[0]:
            mess = mess + "\t" + act
        mess = mess + "\n" + "Dominant Inhib.: "
        for act in why[1]:
            mess = mess + "\t" + act
        mess = mess + "\n" + "Essential Activ.: "
        for act in why[2]:
            mess = mess + "\t" + act
        mess = mess + "\n" + "Dominant Activ.: "
        for act in why[3]:
            mess = mess + "\t" + act
        mess = mess + '\n'
        text_buffer.set_text(mess)
        text.set_editable(False)

        #radio button
        rname = 'rb' + str(self.compt)
        rbut = wtree.get_widget("rButton")
        rbut.set_name(rname)

        if group:
            rbut.set_group(group)
        else :
            self.button_group = rbut
            rbut.set_active(True)

        rbut.connect("toggled", self.rb_callback, self.compt)
        self.compt += 1
        return frame


    def create_solution_file(self, sol_file, all_bool):
        """
        Print solutions in a file
        """
        if all_bool:
            scc_places = self.l_solutions
        else :
            scc_places = [self.l_solutions[self.choice]]

        sfile = open(sol_file,'w')
        for i in range(len(scc_places)):
            pair = scc_places[i]
            mess = '\nGene:'
            mess = mess + '\t' + pair[0]
            why = pair[1]
            mess = mess + "\n" + "Isolated Inhib.:"
            for act in why[0]:
                mess = mess + "\t" + act
            mess = mess + "\n" + "Dominant Inhib.:"
            for act in why[1]:
                mess = mess + "\t" + act
            mess = mess + "\n" + "Essential Activ.:"
            for act in why[2]:
                mess = mess + "\t" + act
            mess = mess + "\n" + "Dominant Activ.:"
            for act in why[3]:
                mess = mess + "\t" + act
            mess = mess + '\n'
            sfile.write(mess)
            sfile.write('\n-------------------------------------------\n\n\n')
        sfile.close()


class STATWindow(BAGWindow):
    """
    Class for displaying statistics
    """
    def __init__(self, stat, emvc, reporter, parent):
        """
        @param l_fsol: list<list<string>>
        @param emvc: edit_mvc
        @param reporter: Classical reporter
        @param parent: parent widget
        """
        BAGWindow.__init__(self, stat, emvc, reporter, parent)

        self.window.set_title("Statistics - "+self.emvc.model.name)
        button = self.wtree.get_widget("but_sv_choice")
        hbbox = self.wtree.get_widget("hbuttonbox1")
        hbbox.remove(button)

    def display_info(self):
        """
        ???
        """
        # info label
        info = self.wtree.get_widget("label_nbsol")
        info.set_text(
            "Model Informations\n"
            "(Please note that some elements may have more than one locations; "
            "take a look at /tmp/logs/cadbiom*)"
        )

        self.frame_sol =  self.wtree.get_widget("frame_sol")
        # button frame
        vbox = gtk.VBox(False, 0)
        self.frame_sol.add(vbox)
        vbox.show()

        scroll = gtk.ScrolledWindow()
        vbox.pack_start(scroll)

        vbox2 = gtk.VBox()
        scroll.add_with_viewport(vbox2)
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.button_group = None

        frame = self.build_button_frame(self.l_solutions)
        vbox2.pack_start(frame)

    def build_button_frame(self, stat_str, group = None):
        """
        ???
        """
        template = pkg_resources.resource_filename(
            __name__,
            "../chart_glade/button_frame.glade"
        )
        wtree = gtk.glade.XML(template)
        wid = wtree.get_widget("button_frame")
        frame = wid.get_child()
        wid.remove(frame)

        #textview
        text = wtree.get_widget("textview")

        text_buffer = text.get_buffer()
        mess = str(stat_str) + '\n'
        text_buffer.set_text(mess)
        text.set_editable(False)

        #radio button removed
        rname = 'rb' + str(self.compt)
        rbut = wtree.get_widget("rButton")
        hbox = wtree.get_widget("hbox")
        hbox.remove(rbut)

        return frame


    def create_solution_file(self, sol_file, all_bool):
        """
        Print solutions in a file
        """
        sfile = open(sol_file,'w')
        mess = self.l_solutions
        mess = mess + '\n'
        sfile.write(mess)
        sfile.write('\n-------------------------------------------\n\n\n')
        sfile.close()



## Filename    : chart_controler.py
## Author(s)   : Michel Le Borgne
## Created     : 4/3/2010
## Revision    :
## Source      :
##
## Copyright 2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided hereunder is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE
##
##
##     http:
##     mailto:
##
## Contributor(s): Geoffroy Andrieux, Nolwenn Le Meur
##
"""
Main gui controler with auxiliary classes
"""
import itertools as it
from string import ascii_uppercase
from math import sqrt
from gtk.gdk import Cursor, ARROW, BOTTOM_LEFT_CORNER, BOTTOM_RIGHT_CORNER,\
                    TOP_LEFT_CORNER, TOP_RIGHT_CORNER, LINE_ON_OFF_DASH
from cadbiom_gui.gt_gui.graphics.drawing_style import Arrow


import gtk

LSIGNALS = ["current_change", "edit_node"]

class ChartClipboard(object):
    """
    A clipboard to be shared by different controlers
    """
    def __init__(self):
        self.clip_node = None

    def put_node(self, node):
        """
        register a chart model node in the clipboard
        """
        self.clip_node = node

    def get_node(self):
        """
        retrieve the current node in the clipboard
        """
        node = self.clip_node
        self.clip_node = None
        return node

    def has_element(self):
        """
        test if the clipboard has a registered node
        """
        if self.clip_node:
            return True
        else:
            return False

    def has_macro(self):
        """
        test if the clipboard has a registered macro node
        """
        if  self.clip_node:
            return self.clip_node.is_macro()
        else:
            return False

class ChartControler(object):
    """
    Implement a controler for graphical views
    """

    cursors = []
    cursors.append(Cursor(ARROW))
    cursors.append(Cursor(TOP_LEFT_CORNER))
    cursors.append(Cursor(TOP_RIGHT_CORNER))
    cursors.append(Cursor(BOTTOM_RIGHT_CORNER))
    cursors.append(Cursor(BOTTOM_LEFT_CORNER))

    def __init__(self, model, cli):
        self.model = model
        # mouse role
        self.mouse_role = "select"
        self.current_node = None
        self.current_handle = 0
        # coordinate of last click in virtual screen 1x1
        self.m_vscreen_coord = None
        self.current_transition = None
        self.lastx = 0 # coordinates of last click in view
        self.lasty = 0
        self.clipboard = cli
        self.drawing_style = None
        self.signal_dict = dict()
        for sig in LSIGNALS:
            self.signal_dict[sig] = []
        self.edit_window = None

        self.gen_name = self.nodes_names_generator()

    def attach(self, signal, obs):
        """
        Register an observer on a signal list
        @param signal: name of the signal
        @param obs: the observer
        """
        lobs = self.signal_dict[signal]
        if not obs in lobs:
            lobs.append(obs)

    def detach(self, signal, obs):
        """
        Remove an observer from a signal list
        """
        self.signal_dict[signal].remove(obs)

    def notify(self, signal):
        """
        Emit a signal
        @param signal: name of the signal
        """
        lobs = self.signal_dict[signal]
        if signal == "current_change":
            for obs in lobs:
                obs.update(self.current_node, self.current_transition)
        elif signal == "edit_node":
            for obs in lobs:
                obs.update(self.current_node)
        else:
            raise Exception("Chart_controler: Unknown signal")

    def set_view(self, view):
        """
        As it says
        """
        self.view = view

    def set_mouse_role(self, role):
        """
        As it says
        """
        self.mouse_role = role


    def button_press(self, widget, event):
        """
        Action when a mouse button is pressed
        """
        self.lastx = event.x
        self.lasty = event.y

        # cursor coordinates in virtual screen 1x1
        xloc = event.x/self.view.draw_width
        yloc = event.y/self.view.draw_height
        self.m_vscreen_coord = (xloc, yloc)

        # we are always in a node, which one?
        if self.current_node:
            self.current_node.selected = False
        if self.current_transition:
            self.current_transition.selected = False

        (node, handle, center, trans) = self.model.find_element(
                                            self.m_vscreen_coord,
                                            self.drawing_style)
        # we found the root of the sub_model - no resizing
        if node == self.model.get_root():
            handle = 0
        self.current_handle = handle
        self.current_node = node
        node.selected = True
        self.current_node_center = center
        if trans:
            node.selected = False
            trans.selected = True
            self.current_transition = trans
        else:
            self.current_transition = None

        # notify what we have found (red color for selected element)
        self.notify("current_change")
        self.model.notify()
        # action (depending on button and event type)
        if event.type == gtk.gdk._2BUTTON_PRESS: # double click desactivated
            return
#            self.mouse_role = 'select'
#            if self.current_node:
#                if self.current_node.is_macro():
#                    self.notify("edit_node")

        if event.button == 1:
            if self.mouse_role == 'select':
                if node: # got a node
                    if handle != 0: # we get a handle => resizing
                        self.mouse_role = 'resizing'
                    else:
                        self.mouse_role = 'moving'
            else:
                pass
        elif event.button == 3:
            self.context_menu(event)


    def context_menu(self, event):
        """
        Menu appearing on right click
        """
        menu = gtk.Menu()
        node = self.current_node
        transition = self.current_transition
        # Create menu-item copy for nodes only
        if not transition:
            buf = "Copy"
            menu_items = gtk.MenuItem(buf)
            menu.append(menu_items)
            menu_items.connect("activate", self.menuitem_response,
                               buf, node, transition)
            menu_items.show()
        # Create menu-item cut
        if transition or node != self.model.get_root():
            buf = "Cut"
            menu_items = gtk.MenuItem(buf)
            menu.append(menu_items)
            menu_items.connect("activate", self.menuitem_response,
                               buf, node, transition)
            menu_items.show()
        # Create menu-item paste
        buf = "Paste"
        menu_items = gtk.MenuItem(buf)
        menu.append(menu_items)
        if self.clipboard.has_element():
            menu_items.connect("activate", self.menuitem_response,
                               buf, node, transition)
        else:
            menu_items.set_sensitive(False)
        menu_items.show()
        # Create menu item paste component
        buf = "Paste components"
        menu_items = gtk.MenuItem(buf)
        menu.append(menu_items)
        if self.clipboard.has_macro():
            menu_items.connect("activate", self.menuitem_response,
                                buf, node, transition)
        else:
            menu_items.set_sensitive(False)
        menu_items.show()

        menu.popup(None, None, None, 3, event.time)

    def menuitem_response(self, widget, buf, node, transition):
        """
        Callback for the preceding menu
        """
        if buf == 'Copy':
            if not transition:
                cnode = node.copy()
                self.clipboard.put_node(cnode)
        elif buf == 'Cut':
            if transition:
                transition.remove()
                node.model.notify()
            else:
                node.remove()
                self.clipboard.put_node(node)
        elif buf == 'Paste':
            if self.clipboard.has_element() and node.is_macro():
                pnode = self.clipboard.get_node()
                node.sub_nodes.append(pnode)
                pnode.father = node
                pnode.set_model(node.model)
                # renumber start and trap nodes (add a number)
                if pnode.is_start() or pnode.is_trap():
                    pnode.name = pnode.name+"%s" % node.count
                    node.count = node.count + 1
                # coordinates in new node
                pnode.xloc = self.m_vscreen_coord[0]
                pnode.yloc = self.m_vscreen_coord[1]
                node.model.modified = True
                node.model.notify()
        elif buf == 'Paste components':
            if self.clipboard.has_element() and node.is_macro():
                pnode = self.clipboard.get_node()
                if pnode.is_macro():
                    # add nodes
                    for snode in pnode.sub_nodes:
                        node.sub_nodes.append(snode)
                        snode.father = node
                        snode.set_model(node.model)
                        # renumber start and trap nodes (add a number)
                        if snode.is_start() or snode.is_trap():
                            snode.name =  snode.name+"%s" % node.count
                            node.count = node.count + 1
                    # add transition groups
                    for gtr in pnode.transitions:
                        node.transitions.append(gtr)
                        for trans in gtr:
                            trans.macro_node = node
                    node.model.modified = True
                    node.model.notify()

    def button_release(self, widget, event):
        """
        callback when a mouse button is released
        """
        # rubout last click characteristics
        if self.mouse_role == 'select':
            return

        elif self.mouse_role == "resizing" or self.mouse_role == "moving" :
            self.view.window.set_cursor(ChartControler.cursors[0])
            self.mouse_role = "select"

        elif self.mouse_role == "new_trans":
            if self.current_node:
                self.new_transition(event.x, event.y)

        else: # assume node creation
            if self.current_node:
                if self.current_node.is_macro():
                    self.new_node(self.mouse_role)

    def new_node(self, type):
        """
        creation of a new node
        @param type: type of the node (string)
        """
        xnode = self.m_vscreen_coord[0]
        ynode = self.m_vscreen_coord[1]
        if type == 'simple':
            self.current_node = self.current_node.add_simple_node(
                next(self.gen_name),
                xnode, ynode)
        elif type == 'macro':
            self.current_node = self.current_node.add_macro_subnode(
                next(self.gen_name),
                xnode,
                ynode,
                0.25, 0.25)
        elif type == 'start':
            self.current_node = self.current_node.add_start_node(xnode, ynode)
        elif type == 'trap':
            self.current_node = self.current_node.add_trap_node(xnode, ynode)
        elif type == 'perm':
            self.current_node = self.current_node.add_perm_node(
                next(self.gen_name),
                xnode, ynode)
        elif type == 'input':
            self.current_node = self.current_node.add_input_node(
                next(self.gen_name),
                xnode, ynode)
        elif type == 'env':
            self.current_node = self.current_node.add_env_node(
                next(self.gen_name),
                xnode, ynode,
                0.25, 0.25)
        else: # bug!
            print ' new_node:UNKNOWN TYPE:', type
            return
        self.current_transition = None
        self.current_handle = 0
        self.m_vscreen_coord = (0.0, 0.0)
        self.notify("current_change")
        self.view.window.set_cursor(ChartControler.cursors[0])
        self.mouse_role = "select"

    def nodes_names_generator(self):
        """Return a generator of names for new nodes.

        Names are generated in lexicographic order.
        """
        for size in it.count(start=1):
            for tpl in it.combinations(ascii_uppercase, size):
                yield "".join(tpl)

    def new_transition(self, xmo, ymo):
        """
        @param xmo, ymo: int mouse screen coordinates
        """
        # are we in a node?
        xloc = xmo / self.view.draw_width
        yloc = ymo / self.view.draw_height
#        w_coef = 1.0/self.view.draw_width
#        h_coef = 1.0/self.view.draw_height
        (node, handle, center, trans) = self.model.find_element((xloc, yloc),
                                                            self.drawing_style)
        if not node: # no extremity (never happens)
            self.view.window.set_cursor(ChartControler.cursors[0])
            self.mouse_role = "select"
            self.model.notify() # rubout the draft transition
            return
        # are we in same container
        if self.current_node.father != node.father:
            self.view.window.set_cursor(ChartControler.cursors[0])
            self.mouse_role = "select"
            self.model.notify() # rubout the draft transition
            return
        # we are in extremity node - current node is origin node
        # is origin node correct and extremity node correct
        if self.current_node.is_for_origin() and node.is_for_extremity():
            trans = self.current_node.father.add_transition(self.current_node,
                                                          node)
            if trans:
                self.current_node.selected = False
                self.current_node = None
                self.m_vscreen_coord = (0.0, 0.0)
                self.current_transition = trans
                trans.selected = True
                self.notify("current_change")
                self.model.notify()
                self.view.window.set_cursor(ChartControler.cursors[0])
                self.mouse_role = "select"
            else:
                self.view.window.set_cursor(ChartControler.cursors[0])
                self.mouse_role = "select"
                self.model.notify() # rubout the draft transition
        else:
            self.view.window.set_cursor(ChartControler.cursors[0])
            self.mouse_role = "select"
            if self.current_node:
                self.current_node.selected = False
            self.model.notify() # rubout the draft transition
            return


    def motion_notify(self, widget, event):
        """
        Callback when moving window
        """
        swi = self.view.draw_width
        she = self.view.draw_height

        # transform mouse coordinates to virtual screen 1.0 x 1.0
        xvirt = event.x / swi
        yvirt = event.y / she

        if self.mouse_role == 'select':
            (node, handle, center, trans) = self.model.find_element(
                                                            (xvirt, yvirt),
                                                            self.drawing_style)
            if node == self.model.get_root():
                handle = 0
            # detect a new handle
            if handle != self.current_handle:
                self.current_handle = handle
                self.view.window.set_cursor(ChartControler.cursors[handle])

        elif self.mouse_role == 'resizing':
            self.current_node.resize(xvirt, yvirt, self.current_handle,
                                     swi, she, self.model.get_root())

        elif self.mouse_role == 'moving':
            v_dx = xvirt - self.m_vscreen_coord[0]
            v_dy = yvirt - self.m_vscreen_coord[1]
            v_size = self.current_node.accept(self.drawing_style)
            self.current_node.move(v_dx, v_dy, v_size, self.model.get_root())
            self.m_vscreen_coord = (xvirt, yvirt)
        elif self.mouse_role == 'new_trans':
            if self.current_node:
                self.draft_transition(event.x, event.y)
        else:
            pass


    def draft_transition(self, xmo, ymo):
        """
        draw a transition arrow in dotted line
        @param xmo,ymo: mouse coordinates
        """
        view = self.view
        self.model.notify()
        xr1 = int(self.current_node_center[0]*self.view.draw_width)
        yr1 = int(self.current_node_center[1]*self.view.draw_height)

        # graphic context
        pixmap = view.pixmap
        grc = view.window.new_gc()
        grc.set_line_attributes(1, LINE_ON_OFF_DASH, 0, 0)
        # line
        pixmap.draw_line(grc, xr1, yr1, int(xmo), int(ymo))
        # arrow
        unx = xmo - xr1
        uny = ymo - yr1
        norm = sqrt(unx ** 2 + uny ** 2)
        if norm != 0:
            unx = unx / norm
            uny = uny / norm
            arr = Arrow()
            arr.draw(view, (unx, uny), (xmo, ymo))
        grc = view.window.new_gc()
        view.window.draw_drawable(grc, view.pixmap, 0, 0, 0, 0,
                                  view.draw_width, view.draw_height)

#    def edit_constraints(self):
#        lang = 'biosignal'
#        ed_conf = TextEditConfig()
#        self.edit_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
#        self.edit_window.connect("destroy", self.save_constraints)
#        self.edit_window.set_title("Constraints: "+self.model.name)
#        self.edit_window.set_position(gtk.WIN_POS_CENTER)
#        self.edit_window.set_keep_above(True)
#        self.edit_window.set_default_size(600,800)
#
##        scroll = gtk.ScrolledWindow()
##        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
##        scroll.set_shadow_type(gtk.SHADOW_IN)
##        self.edit_window.add(scroll)
#
#        self.text_page = TextPage(lang, ed_conf)
#        self.text_page.set_text(self.model.constraints)
#        self.edit_window.add(self.text_page)
#        self.edit_window.show_all()
#
#    def save_constraints(self, widget):
#        text = self.text_page.get_text()
#        self.model.constraints = text
#        self.edit_window.destroy()


# API for communication with charter
    def set_action_select(self, widget):
        """
        As it says
        """
        self.mouse_role = "select"

    def set_action_new_simple_node(self, widget):
        """
        As it says
        """
        self.mouse_role = "simple"

    def set_action_new_macro_node(self, widget):
        """
        As it says
        """
        self.mouse_role = "macro"

    def set_action_new_start_node(self, widget):
        """
        As it says
        """
        self.mouse_role = "start"

    def set_action_new_trap_node(self, widget):
        """
        As it says
        """
        self.mouse_role = "trap"

    def set_action_new_input_node(self, widget):
        """
        As it says
        """
        self.mouse_role = "input"

    def set_action_new_transition(self, widget):
        """
        As it says
        """
        if self.current_node:
            self.current_node.selected = False
        self.current_node = None
        self.current_handle = 0
        self.m_vscreen_coord = None
        if self.current_transition:
            self.current_transition.selected = False
        self.current_transition = None
        self.mouse_role = "new_trans"

    def set_action_new_perm_node(self, widget):
        """
        As it says
        """
        self.mouse_role = "perm"


# controler for navigation view

class NavControler(object):
    """
    Implement a controler for navigation views
    """

    def __init__(self):
        self.lastx = 0 # coordinates of last click in view
        self.lasty = 0
        self.in_ret = False
        self.obs = []

    def set_view(self, view):
        """
        As it says
        """
        # attribute view is assigned when a view is created
        self.view = view

    def attach(self, obs):
        """
        observer management
        """
        if not obs in self.obs:
            self.obs.append(obs)

    def detach(self, obs):
        """
        observer management
        """
        self.obs.remove(obs)

    def notify(self, depx, depy):
        """
        observer management
        """
        for obs in self.obs:
            obs.update(depx, depy)

    def button_press(self, widget, event):
        """
        action when a mouse button is pressed
        """
        self.lastx = event.x
        self.lasty = event.y
        cond = self.view.x_ret <= self.lastx
        cond = cond and (self.lastx <= (self.view.x_ret + self.view.w_ret))
        if cond :
            cond2 = (self.view.y_ret <= self.lasty)
            vvv = (self.view.y_ret + self.view.h_ret)
            cond2 = cond2 and (self.lasty <= vvv)
            if cond2:
                self.in_ret = True

    def button_release(self, widget, event):
        """
        action when a mouse button is released
        """
        self.in_ret = False
        return

    def motion_notify(self, widget, event):
        """
        callback
        """
        if self.in_ret:
            depx = event.x - self.lastx
            depy = event.y - self.lasty
            alloc = widget.get_allocation()
            # limits
            vv1 = self.view.x_ret + depx + self.view.w_ret
            vv2 = self.view.x_ret + depx
            if (vv1 > alloc.width) or (vv2 < 0):
                depx = 0
            vv1 = self.view.y_ret + depy + self.view.h_ret
            vv2 = self.view.y_ret + depy
            if (vv1 > alloc.height) or (vv2 < 0):
                depy = 0
            self.lastx = self.lastx + depx
            self.lasty = self.lasty + depy
            # reticule move
            if (not depx == 0) or  (not depy == 0):
                depx = float(depx) / alloc.width
                depy = float(depy) / alloc.height
                self.notify(depx, depy)








## Filename    : chart_simul_controler.py
## Author(s)   : Michel Le Borgne
## Created     : 04/2010
## Revision    :
## Source      :
##
## Copyright 2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License  published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided here under is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE
##
##
## Contributor(s): Geoffroy Andrieux, Nolwenn Le Meur
##
"""
Controller and auxiliary widgets for simulations with gui.
"""
import gtk
from time import clock


from chart_chrono import ChartChrono
from cadbiom_gui.gt_gui.utils.warn import cancel_warn
from cadbiom.models.guard_transitions.simulator.simul_exceptions import \
                              SimulException, SimulStopException
from cadbiom.models.guard_transitions.simulator.chart_simul import ChartSimulator
from cadbiom.models.guard_transitions.simulator.simul_aux import ModelExtractorVisitor
from cadbiom.models.guard_transitions.analyser.ana_visitors import FrontierVisitor
from cadbiom_gui.gt_gui.utils.listDisplay  import ToggleList
from cadbiom_gui.gt_gui.utils.fileHandling import FileChooser

import pkg_resources

class ChartSimulControler(object):
    """
    Control different parameters of a simulation and display panel
    """
    def __init__(self, emvc, sim_option, reporter,
                 init_places = None, icseq=None, parent = None):
        # loading and initializing the simulator model
        self.edit_mvc = emvc
        self.model = emvc.model                    # chart model
        self.option = sim_option                   # free clock treatment option
        self.reporter = reporter                   # display error messages
        self.sim = ChartSimulator()                # simulation model
        self.__reload = True
        if icseq: # simulation with conditions from checker
            self.sim.set_act_input_stream(icseq)
            self.__reload = False
        if init_places:
            self.__reload = False
        self.sim.build(self.model, sim_option, reporter)
        if reporter.error:
            DisplayError(reporter, self.model.name)
            return

        self.init_places = init_places
        if self.init_places:
            self.sim.simul_init_places(self.init_places)
        else:
            self.sim.simul_init()

        self.max_step = 0
        self.tempo = 0.0
        self.stop = False
        self.selected_chrono = False
        self.main_window = None
        self.selected_window = None

        # graphical interface
        # Set the Glade
        template = pkg_resources.resource_filename(
            __name__,
            "../chart_glade/chart_simulator.glade"
        )
        self.wtree = gtk.glade.XML(template)

        # Get the Main Window, and connect the "destroy" event
        self.main_window = self.wtree.get_widget("window1")
        self.main_window.set_title("Simulation: "+self.model.name)
        if (self.main_window):
            self.main_window.connect("destroy", self.on_destroy)
        self.main_window.set_position(gtk.WIN_POS_CENTER)
        self.main_window.set_keep_above(True)

        # frontier button (disconnected when icseq are computed)
        button = self.wtree.get_widget("frontier_button")
        button.connect("clicked", self.on_select_front)
        if icseq:
            button.set_sensitive(False)

        # whole button (disconnected when ic are computed)
        button = self.wtree.get_widget("whole_button")
        button.connect("clicked", self.on_select_whole)
        if icseq:
            button.set_sensitive(False)

        # start button
        button = self.wtree.get_widget("startbutton")
        button.connect("clicked", self.on_start)

        # chrono nutton
        button = self.wtree.get_widget("chronobutton")
        button.connect("clicked", self.on_chrono)

        # input file choser (disconnected when ic are computed)
        button = self.wtree.get_widget("input_file_button")
        if not self.sim.has_input():
            button.set_sensitive(False)
        button.connect("clicked", self.on_input_file_selected)
        if icseq:
            button.set_sensitive(False)

        # step button
        button = self.wtree.get_widget("stepbutton")
        button.connect("clicked", self.on_step_simul)


        # check button for chronogram
        self.cb_chrono = self.wtree.get_widget("cb_chrono")

        # nb of steps widget
        self.nb_step_entry = self.wtree.get_widget("nbsteps")

        # slider
        self.range = self.wtree.get_widget("hscale1")
        self.range.set_update_policy(gtk.UPDATE_DISCONTINUOUS)
        self.range.connect("value_changed", self.on_delay)

        # init button"
        button = self.wtree.get_widget("initbutton")
        button.connect("clicked", self.on_init)

        # reload button (disconnected when using computed input clocks)
        button = self.wtree.get_widget("reloadbutton")
        button.connect("clicked", self.on_reload)
        if not self.__reload:
            button.set_sensitive(False)

        # extract button
        button = self.wtree.get_widget("extractbutton1")
        button.connect("clicked", self.on_extract, False)
        button = self.wtree.get_widget("extractbutton2")
        button.connect("clicked", self.on_extract, True)

        # register itself for parent or emvc
        if parent:
            parent.win_register(self)
        else:
            self.edit_mvc.win_register(self)
        self.parent = parent

        # display
        self.main_window.show_all()
        #gtk.main()

    def on_destroy(self, widget):
        """
        destroy the windows after all secondary windows
        """
        if self.selected_window:
            self.selected_window.destroy()
        if self.sim.chrono:
            self.sim.chrono.window.destroy()
        self.model.clean()
        self.model.notify()
        if self.parent:
            self.parent.win_remove(self)
        else:
            self.edit_mvc.win_remove(self)
        if self.main_window:
            self.main_window.destroy()

    def destroy(self):
        """
        Used if chart_simul_controller is itself a secondary window
        """
        self.on_destroy(None)

    def on_step_simul(self, widget):
        """
        callback for step by step simulation
        """
        # in case a selected chrono is open"
        if self.selected_chrono:
            self.sim.chrono.destroy()
            self.sim.chrono = None
            self.selected_chrono = False
        if self.sim.has_input() and not self.sim.input_buffer:
            # inputs present and no input file
            cancel_warn("Set input file before simulation")
            return
        try:
            self.sim.simul_step()
            if self.sim.chrono:
                self.sim.chrono.register(self.sim)
                self.sim.chrono.update()
        except SimulStopException:
            cancel_warn("Simulation reached end of input")
        except SimulException , exc:
            cancel_warn("Problem in simulation: " + exc.__str__())


    def on_input_file_selected(self, widget):
        """
        chose an input file
        """
        fch = FileChooser("Input file", "", "*", save_window=False)
        fch.do_action(self.set_input_file)

    def set_input_file(self, filename):
        """
        connect an input file
        """
        inputfile = open(filename, 'r')
        fline = inputfile.readline()
        line_split = fline.split()
        inputfile.close()
        # test type of file on first line
        if line_split[0] == "$inputs": # old coded input
            try:
                self.sim.set_cfile_input_stream(filename)
            except Exception , exc:
                cancel_warn("Error in coded input file loading" + exc.__str__())
            return
        elif  line_split[0] == "%": # activated input
            try:
                self.sim.set_act_file_input_stream(filename)
            except Exception , exc:
                mess = "Error in active input file loading" + exc.__str__()
                cancel_warn(mess)
            return
        else: # assume scenario
            try:
                self.sim.set_sce_file_input_stream(filename)
            except Exception , exc:
                cancel_warn("Error in scenario loading" + exc.__str__())
            return


    def on_init(self, widget):
        """
        initialisation of the simulator
        """
        if self.sim.init_places:
            self.sim.simul_init_places(self.sim.init_places)
        else:
            self.sim.simul_init() # standard init from graphics

        if self.sim.input_buffer:
            self.sim.input_buffer.rewind()
        if self.sim.chrono:
            if self.selected_chrono:
                self.sim.chrono.window.destroy()
                self.sim.chrono = None
                self.selected_chrono = False
            else:
                self.sim.chrono.rewind()
                self.sim.chrono.register(self.sim)
                self.sim.chrono.update()

    def on_start(self, widget):
        """
        start a simulation
        """
        # check inputs
        if self.sim.has_input() and not self.sim.input_buffer:
            # inputs present and no input file
            cancel_warn("Set input file before simulation")
            return
        # read nb of steps
        max_step = self.nb_step_entry.get_text()
        if max_step == "":
            self.max_step = 0
            cancel_warn("Set number of simulation steps")
        else:
            self.max_step = int(max_step)
        self.step_count = 0
        self.stop = False
#        a = threading.Thread(None, self.loop, None, (), None)
#        a.start()
#        return
        # selected chrono
        if self.cb_chrono.get_active():
            if self.sim.chrono:
                self.sim.chrono.destroy()
            self.sim.chrono = ChartChrono.for_trajectory(self.sim)
            self.selected_chrono = True
        self.loop()
        if self.selected_chrono:
            self.sim.chrono.display_selected_chrono()

    def loop(self):
        """
        temporized loop for automatic simulation
        """
        if self.sim.chrono:
            self.sim.chrono.register(self.sim)
        while not self.stop and self.step_count < self.max_step:
            start_time = clock()
            try:
                self.sim.simul_step()
                self.step_count = self.step_count + 1
            except SimulStopException:
                cancel_warn("Simulation reached end of input")
                return
            except SimulException , exc:
                cancel_warn("Problem in simulation:" + exc.__str__())
                return
            self.model.notify()
            if self.sim.chrono:
                self.sim.chrono.register(self.sim)
                self.sim.chrono.update()
            # tempo
            if self.tempo > 0.01:
                time = clock()
                delay = time - start_time
                while delay < self.tempo:
                    time = clock()
                    delay = time - start_time

    def on_chrono(self, widget):
        """
        setup chronogram
        """
        if not self.sim.chrono:
            # logical analyser
            self.sim.chrono = ChartChrono.from_simulation_model(self.sim)
            self.sim.chrono.register(self.sim)
            self.sim.chrono.update()
        else:
            self.sim.chrono.destroy()
            self.sim.chrono = None
            self.selected_chrono = False

    def on_delay(self, widget):
        """
        adjust the simulation timer
        """
        self.tempo = self.range.get_value()/50.0

    def on_reload(self, widget):
        """
        reset the simulator
        """
        if not self.__reload:
            cancel_warn("No reload allowed")
            return
        if self.sim.chrono:
            self.sim.chrono.destroy()
        input_file = self.sim.input_file
        self.sim = ChartSimulator()
        self.reporter.error = False
        self.sim.build(self.model, self.option, self.reporter)
        if self.reporter.error:
            DisplayError(self.reporter, self.model)
            return
        if input_file:
            try:
                self.set_input_file(input_file)
            except SimulException, exc:
                self.sim.input_buffer = None
                cancel_warn ("old input file inadequate - redefine input")
        if self.sim.chrono:
            self.sim.chrono.destroy()
            self.sim.chrono = ChartChrono.from_simulation_model(self.sim)
        if self.init_places:
            self.sim.simul_init_places(self.init_places)
        else:
            self.sim.simul_init() # standard init from graphics
        if self.sim.chrono:
            self.sim.chrono.register(self.sim)
            self.sim.chrono.update()
        self.max_step = 0
        self.tempo = 0.0
        self.stop = False

    def on_extract(self, widget, ex_type):
        """
        extract the part of a model used in last simulation
        """
        if self.sim.get_step() < 2:
            return
        mex = ModelExtractorVisitor('sim_extract', ex_type)
        self.model.accept(mex)
        model = mex.sub_model
        self.edit_mvc.charter.add_display_mvc(model.name, model, False)
        self.edit_mvc.charter.do_layout(None,"hierarchical_LR")

    def on_select_front(self, widget):
        """
        selection of frontier activated places at initialization
        """
        self.selected_window = SelectPlaceWindow(self, True)

    def on_select_whole(self, widget):
        """
        selection of activated places at initialization
        """
        self.selected_window = SelectPlaceWindow(self, False)




class DisplayError(object):
    """
    auxiliary widget for simulator
    """
    def __init__(self, report, model_name, text=None):
        self.main_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        #self.main_window.connect("destroy", gtk.main_quit)
        self.main_window.set_title("Compiler errors or info: " + model_name)
        self.main_window.set_position(gtk.WIN_POS_CENTER)
        self.main_window.set_keep_above(True)
        self.main_window.set_default_size(500, 300)

        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scroll.set_shadow_type(gtk.SHADOW_IN)
        self.main_window.add(scroll)

        write = gtk.TextView()
        if report:
            write.get_buffer().set_text(report.memory)
        else:
            write.get_buffer().set_text(text)
        scroll.add_with_viewport(write)
        self.main_window.show_all()
        #gtk.main()

class SelectPlaceWindow(object):
    """
    Select the activated place for initialization
    """
    def __init__(self,  simulator, frontier_bool):

        self.simulator = simulator
        # graphical interface
        # Set the Glade file
        template = pkg_resources.resource_filename(
            __name__,
            "../chart_glade/simu_places.glade"
        )
        self.wtree = gtk.glade.XML(template)

        # Get the Main Window, and connect the "destroy" event
        self.window = self.wtree.get_widget("window1")
        self.window.set_title("Selected window")
        self.window.set_size_request(250, 400)
        self.window.set_position(gtk.WIN_POS_NONE)
        self.window.set_keep_above(True)

        place_notebook = self.wtree.get_widget("place_notebook")
        # search
        if frontier_bool:
            self.search_area = ToggleFrontier(self.simulator.model,
                                              place_notebook, "Frontier")
        else:
            self.search_area = ToggleWholeModel(self.simulator.model,
                                                place_notebook,
                                                "Simple node list")


        # button
        button = self.wtree.get_widget("select_button")
        button.connect("clicked", self.on_select)
        button = self.wtree.get_widget("unselect_button")
        button.connect("clicked", self.on_unselect)
        button = self.wtree.get_widget("save_button")
        button.connect("clicked", self.on_save)
        button = self.wtree.get_widget("load_button")
        button.connect("clicked", self.on_load)
        button = self.wtree.get_widget("validate_button")
        button.connect("clicked", self.on_validate)

        # display
        self.window.show_all()


    def destroy(self):
        """
        as usual for aux widget
        """
        self.window.destroy()

    def on_select(self, widget):
        """
        ???
        """
        self.search_area.sn_viewer.select_all(widget)

    def on_unselect(self, widget):
        """
        ???
        """
        self.search_area.sn_viewer.clear(widget)

    def on_save(self, widget):
        """
        open a window to save initialization
        """
        choice = gtk.FileChooserDialog("Save initialization", None,
                                       gtk.FILE_CHOOSER_ACTION_SAVE,
                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                        gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        choice.set_default_response(gtk.RESPONSE_OK)

        #add a filter to see only txt files
        filter = gtk.FileFilter()
        filter.set_name("text files")
        filter.add_pattern("*.txt")
        choice.add_filter(filter)

        #add a filter to see all
        no_filter = gtk.FileFilter()
        no_filter.set_name("all")
        no_filter.add_pattern("*")
        choice.add_filter(no_filter)

        response = choice.run()
        if response == gtk.RESPONSE_OK:
            self.create_init_file(choice.get_filename())
        elif response == gtk.RESPONSE_CANCEL:
            pass
            #print 'Closed, no files selected'
        choice.destroy()

    def on_load(self, widget):
        """
        open a window to search xml file coming from PID database
        """
        choice = gtk.FileChooserDialog("Import from xml", None,
                                       gtk.FILE_CHOOSER_ACTION_OPEN,
                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                        gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        choice.set_default_response(gtk.RESPONSE_OK)

        #add a filter to see only xml files
        filter = gtk.FileFilter()
        filter.set_name("text files")
        filter.add_pattern("*.txt")
        choice.add_filter(filter)

        #add a filter to see all
        no_filter = gtk.FileFilter()
        no_filter.set_name("all")
        no_filter.add_pattern("*")
        choice.add_filter(no_filter)

        response = choice.run()
        if response == gtk.RESPONSE_OK:
            self.load_init_file(choice.get_filename())
        elif response == gtk.RESPONSE_CANCEL:
            #print 'Closed, no files selected'
            pass
        choice.destroy()

    def on_validate(self, widget):
        """
        validate the choice of activated places
        """
        self.set_places()
        self.destroy()

    def create_init_file(self, file_name):
        """
        make a txt file with the current selected places
        """
        file = open(file_name,'w')
        list_of_places = self.search_area.get_selected_keywords()
        for place in list_of_places :
            file.write(str(place)+'\n')
        file.close()
        return

    def load_init_file(self, file_name):
        """
        load a file to initialize the simulator
        """
        list_of_places = []
        file = open(file_name,'r')
        for line in file:
            name = line[:-1]
            if name not in self.search_area.list_nodes :
                cancel_warn("Name in the file not in the model")
                return
            list_of_places.append(name)
        self.search_area.sn_viewer.set_selected_items(list_of_places)
#        self.simulator.sim.simul_init_places(list_of_places)
#        self.destroy()

    def set_places(self):
        """
        set the chosen activated places
        """
        self.simulator.init_places = self.search_area.get_selected_keywords()
        skw = self.search_area.get_selected_keywords()
        self.simulator.sim.simul_init_places(skw)

class ToggleWholeModel(object):
    """
    List of simple nodes for searching
    """
    def __init__(self, model, notebook, label):
        self.model = model
        self.model_changed = True
        self.notebook = notebook

        # simple nodes
        sn_frame = gtk.Frame()
        vbox = gtk.VBox(False, 0)
        sn_frame.add(vbox)
        # simple nodes list
        self.sn_viewer = ToggleList()
        label = gtk.Label(label)
        # wrap into scrollwindow
        scroll = gtk.ScrolledWindow()
        scroll.add_with_viewport(self.sn_viewer)
        vbox.pack_start(scroll, True, True, 0)

        notebook.append_page(sn_frame, label)

        # get node names
        lnode = model.get_simple_node_names()
        lnode.sort()
        self.list_nodes = lnode
        # display node names
        self.sn_viewer.refresh(lnode)

    def get_selected_keywords(self):
        """
        retrieve the selected items
        """
        return self.sn_viewer.get_selected_items()

class ToggleFrontier(ToggleWholeModel):
    """
    List of frontier nodes for searching
    """
    def __init__(self, model, notebook, label):
        ToggleWholeModel.__init__(self, model, notebook, label)

        self.model = model
        # get node names
        lnode = self.get_frontier_node_names()
        lnode.sort()
        self.list_nodes = lnode
        # display node names
        self.sn_viewer.refresh(lnode)

    def get_frontier_node_names(self):
        """
        retrieve frontier nodes from the model
        """
        fvi = FrontierVisitor()
        self.model.accept(fvi)
        return fvi.frontier



## Filename    : chart_checker_controler.py
## Author(s)   : Michel Le Borgne
## Created     : 05/2011
## Revision    :
## Source      :
##
## Copyright 2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided here under is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE
##
##
## Contributor(s): Geoffroy Andrieux, Nolwenn Le Meur
##
"""
Collection of widgets for properties checking
"""
import gtk

from cadbiom.models.clause_constraints.mcl.MCLAnalyser import MCLAnalyser
from cadbiom_gui.gt_gui.chart_simulator.chart_simul_controler import ChartSimulControler
from cadbiom_gui.gt_gui.chart_simulator.chart_chrono import ChartChrono
from cadbiom_gui.gt_gui.utils.warn import ok_warn, cancel_warn, confirm
from cadbiom.models.guard_transitions.simulator.chart_simul import  ChartSimulator
from cadbiom.models.clause_constraints.mcl.MCLQuery import MCLSimpleQuery
from cadbiom import commons as cm

import pkg_resources

LOGGER = cm.logger()


class ChartChecker(object):
    """
    This class provide a gui interface for checking some queries
    """

    def __init__(self, emvc, reporter, parent=None):
        self.__emvc = emvc                    # edit mvc - link with charter
        self.__reporter = reporter
        self.__occ_form = OccurenceForm(emvc, reporter)
        self.__current_form = self.__occ_form

        # graphical interface
        self.__main_window = gtk.Window()
        self.__main_window.set_title("Property check: " +
                                     self.__emvc.model.name)
        if (self.__main_window):
            self.__main_window.connect("destroy", self.__on_destroy)
        self.__main_window.set_position(gtk.WIN_POS_CENTER)
        self.__main_window.resize(600, 300)

        if parent:
            # Set window above all windows
            self.__main_window.set_transient_for(parent.main_window)

        # Event on_escape key pressed
        self.__main_window.connect('key_press_event', self.on_escape)

        # register as auxiliary window
        self.__emvc.win_register(self)

        # display
        self.__main_window.add(self.__current_form.get_frame())
        self.__main_window.show_all()

    def __on_destroy(self, widget):
        """
        delete window and dependant sub widgets
        """
        self.__current_form.clean_subwin()
        if self.__main_window:
            self.__main_window.destroy()

    def destroy(self):
        """
        call back adaptor
        """
        self.__on_destroy(None)

    def on_escape(self, widget, event):
        """On ESC key_press_event, destroy this window."""
        if gtk.gdk.keyval_name(event.keyval) == "Escape":
            self.__main_window.destroy()


class OccurenceForm(object):
    """
    Former class for occurence checking
    Now general simple query checking
    """
    def __init__(self, emvc, er_rep):
        self.__emvc = emvc
        self.__model = emvc.model  # chart model
        self.__mcla = None         # clause constraint model analyser
        self.__error_reporter = er_rep
        template = pkg_resources.resource_filename(
            __name__,
            "../chart_glade/occurence_form.glade"
        )
        self.__wtree = gtk.glade.XML(template)
        self.__window = self.__wtree.get_widget("windowOcc")
        wid = self.__wtree.get_widget("frameOcc")
        self.__frame = wid.get_child()
        wid.remove(self.__frame)

        # information
        possible = self.__wtree.get_widget("possible")
        possible.set_active(True)
        self.possible = True
        possible.connect("toggled", self.rb_callback, 'p')
        impossible = self.__wtree.get_widget("impossible")
        impossible.connect("toggled", self.rb_callback, 'i')

        self.step_entry = self.__wtree.get_widget("max_step")
        self.step_entry.set_text('10')
        self.step_entry.connect("changed", self.on_entry_changed)
        self.property_entry = self.__wtree.get_widget("property_entry")
        self.property_entry.connect("changed", self.on_entry_changed)
        self.inv_prop_entry = self.__wtree.get_widget("property_entry2")
        self.inv_prop_entry.connect("changed", self.on_entry_changed)
        self.start_prop_entry = self.__wtree.get_widget("property_entry3")
        self.start_prop_entry.connect("changed", self.on_entry_changed)

        # yes/no button
        button = self.__wtree.get_widget("button_yn")
        button.connect("clicked", self.on_yn)
        # conditions button
        button = self.__wtree.get_widget("button_cond")
        button.connect("clicked", self.on_cond)

        # condition exploration
        self.max_sol_entry = self.__wtree.get_widget("entry_nbsol")
        self.max_sol_entry.set_text('10')
        self.max_sol_entry.connect("changed", self.on_entry_changed)
        self.but_solutions = self.__wtree.get_widget("but_solutions")
        self.but_solutions.connect("clicked", self.on_solutions)
        self.but_solutions.set_sensitive(False)

        self.but_cam = self.__wtree.get_widget("but_cam")
        self.but_cam.connect("clicked", self.on_cam)
        self.but_cam.connect("clicked", self.on_entry_changed)
        self.but_cam.set_sensitive(False)

        # children
        self.__aux_win = []

    def get_frame(self):
        """
        as name says
        """
        return self.__frame

    def rb_callback(self, widget, idw):
        """
        possible and impossible call back
        """
        if idw == 'p':
            self.possible = True
        else:
            self.possible = False

    def on_entry_changed(self, widget):
        """
        react in cas of change in entries
        """
        #  buttons
        self.but_solutions.set_sensitive(False)
        self.but_cam.set_sensitive(False)

    def on_yn(self, widget):
        """
        lauch a satisfiability test
        """
        # get info
        final_prop = self.property_entry.get_text()
        if len(final_prop) == 0:
            cancel_warn("Void property")
            return
        inv_prop = self.inv_prop_entry.get_text()
        start_prop = self.start_prop_entry.get_text()
        max_step_str = self.step_entry.get_text()
        if len(max_step_str) == 0:
            cancel_warn("Number of steps not specified")
            return
        max_step = int(max_step_str)
        self.mcla = MCLAnalyser(self.__error_reporter)
        self.mcla.build_from_chart_model(self.__model)
        if self.__error_reporter.error:
            cancel_warn(self.__error_reporter.memory)
            self.__error_reporter.reset()
            return
        # solve
        if self.possible:
            if len(inv_prop) == 0:
                inv_prop = None
            else :
                inv_prop = "not ("+inv_prop+")"

            if len(start_prop) == 0:
                start_prop = None
            query = MCLSimpleQuery(start_prop, inv_prop, final_prop)
            reach = self.mcla.sq_is_satisfiable(query, max_step)

            if self.__error_reporter.error:
                cancel_warn(self.__error_reporter.memory)
                self.__error_reporter.reset()
                return
            if reach:
                ok_warn("Yes, we can!","green")
            else:
                ok_warn("NO","red")
        else:
            if len(inv_prop) != 0:
                final_prop = "not ("+final_prop+" and "+inv_prop+")"

            query = MCLSimpleQuery(start_prop, inv_prop, None)
            inv = self.mcla.sq_is_satisfiable(query, max_step)
            if self.__error_reporter.error:
                cancel_warn(self.__error_reporter.memory)
                self.__error_reporter.reset()
                return
            if inv:
                ok_warn("Yes, we can","green")
            else:
                ok_warn("NO","red")

    def on_cond(self, widget):
        """
        launch solution computations
        """
        # desactivate buttons
        self.but_solutions.set_sensitive(False)
        self.but_cam.set_sensitive(False)

        # get info
        final_prop = self.property_entry.get_text()
        if len(final_prop) == 0:
            cancel_warn("Void property")
            return
        inv_prop = self.inv_prop_entry.get_text()
        start_prop = self.start_prop_entry.get_text()
        max_step_str = self.step_entry.get_text()
        if len(max_step_str) == 0:
            cancel_warn("Number of steps not specified")
            return
        max_step = int(max_step_str)
        max_sol_str = self.max_sol_entry.get_text()
        if len(max_sol_str) == 0:
            cancel_warn("Number of solutions not specified")
            return
        max_sol = int(max_sol_str)

        self.mcla = MCLAnalyser(self.__error_reporter)
        self.mcla.build_from_chart_model(self.__model)
        if self.__error_reporter.error:
            cancel_warn(self.__error_reporter.memory)
            self.__error_reporter.reset()
            return

        # solve
        if self.possible:
            LOGGER.debug("OccurenceForm.on_cond:: Solve is possible !")
            if len(inv_prop) == 0:
                inv_prop = None
            else :
                inv_prop = "not ("+inv_prop+")"

            if len(start_prop) == 0:
                start_prop = None

            query = MCLSimpleQuery(start_prop, inv_prop, final_prop)
            self.lsol = self.mcla.sq_frontier_solutions(query ,
                                                        max_step, max_sol)
            LOGGER.debug("OccurenceForm.on_cond:: Solutions: " + \
                         str(len(self.lsol)))
            if self.__error_reporter.error:
                cancel_warn(self.__error_reporter.memory)
                self.__error_reporter.reset()
                return
        else:
            LOGGER.debug("OccurenceForm.on_cond:: Solve is NOT possible !")
            if len(inv_prop) != 0:
                final_prop = "not ("+final_prop+" and "+inv_prop+")"
            query = MCLSimpleQuery(start_prop, inv_prop, None)
            self.lsol = self.mcla.sq_frontier_solutions(query,
                                                        max_step, max_sol)
            LOGGER.debug("OccurenceForm.on_cond:: Solutions: " + \
                         str(len(self.lsol)))
            if self.__error_reporter.error:
                cancel_warn(self.__error_reporter.memory)
                self.__error_reporter.reset()
                return


        # affectation
        self.property = final_prop
        self.inv_prop = inv_prop
        self.start_prop = start_prop
        self.max_step = max_step
        self.max_sol = max_sol

        if len(self.lsol) == 0:
            cancel_warn("No conditions")
            return

        # activate buttons
        self.but_solutions.set_sensitive(True)
        self.but_cam.set_sensitive(True)

    def on_solutions(self, widget):
        """
        display solutions
        """
        SolutionWindow(self.lsol, self.__emvc, self.__error_reporter, self)

    def on_common(self, widget):
        """
        obsolete
        """
#        common_number=self.mcla.get_common_sol(self.mcla.solutions)
#        if len(common_number)!=0 :
#            common_name=self.mcla.get_state_list_from_solution(common_number)
#            common_length=self.mcla.get_nb_activated(common_number)
#            suff=self.mcla.is_sufficient(common_number, self.inv_prop,
#                                         self.property, self.max_step)
#            Common_Window(common_name, common_length, suff, self.emvc,
#                                         self.__error_reporter, self)
#        else :
#            ok_warn("There is no common solution")
        pass

    def on_cam(self, widget):
        """
        launch mac computations
        """
        ask = confirm(None, 'This process takes time' + '\n' +
                    'You will not be able to use cadbiom.'+'\n' +
                    ' Do you want to continue ?')
        if ask:
            query = MCLSimpleQuery(self.start_prop,
                                   self.inv_prop, self.property)
            cam_list = self.mcla.mac_search(query, self.max_step)
            if len(cam_list)==0 :
                ok_warn("The solver returns an empty list" +
                        "\n"+" you should refine your query")
            else :
                CNSWindow(cam_list, self.__emvc,
                          self.__error_reporter, self)

    # sub windows management
    def win_register(self, win):
        """
        register sub window for destroy
        """
        self.__aux_win.append(win)

    def win_remove(self, win):
        """
        used when the sub window destroys itself
        """
        if win in self.__aux_win:
            self.__aux_win.remove(win)

    def clean_subwin(self):
        """
        destroy all registered subwindows
        """
        for win in self.__aux_win:
            win.destroy()

    def destroy(self):
        """
        destroy all
        """
        self.clean_subwin()
        if self.__window:
            self.__window.destroy()


class SolutionWindow(object):
    """
    Class used for displaying solutions
    """
    def __init__(self, l_fsol, emvc, reporter, parent):
        """
        @param l_fsol: list<FrontierSolution>
        @param emvc: edit_mvc
        @param reporter: Classical reporter
        @param parent: parent widget
        """
        self.emvc = emvc
        self.reporter = reporter
        # frontier place list extraction and ic sequence extraction
        self.l_solutions = []
        self.l_icseq = []
        for fsol in l_fsol:
            self.l_solutions.append(fsol.activated_frontier)
            self.l_icseq.append(fsol.ic_sequence)

        self.aux_win = []
        self.simul = None

        self.compt = 0
        self.choice = 0
        self.textview_list = []

        # window creation
        template = pkg_resources.resource_filename(
            __name__,
            "../chart_glade/checker_solutions.glade"
        )
        self.wtree = gtk.glade.XML(template)
        self.window = self.wtree.get_widget("window1")

        self.window.set_resizable(True)
        height = gtk.gdk.screen_height()
        height = int(height * 0.30)
        self.window.set_size_request(700, height)


        # button save choice
        button = self.wtree.get_widget("but_sv_choice")
        button.connect("clicked", self.on_save, False)

        # button save all
        button = self.wtree.get_widget("but_sv_all")
        button.connect("clicked", self.on_save, True)
        if len(self.l_solutions) == 1:
            button.set_sensitive(False)

        # info label
        info = self.wtree.get_widget("label_nbsol")
        info.set_text(str(len(self.l_solutions))+" solutions found")

        self.frame_sol =  self.wtree.get_widget("frame_sol")
        # button frame
        vbox = gtk.VBox(False, 0)
        self.frame_sol.add(vbox)
        vbox.show()

        scroll = gtk.ScrolledWindow()
        vbox.pack_start(scroll)

        vbox2 = gtk.VBox()
        scroll.add_with_viewport(vbox2)
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.button_group = None

        frame = self.build_button_frame(self.l_solutions[0])
        vbox2.pack_start(frame)

        for sol in self.l_solutions[1:]:
            hsep = gtk.HSeparator()
            vbox2.pack_start(hsep, False, False)

            frame = self.build_button_frame(sol, self.button_group)
            vbox2.pack_start(frame)

        # button inhibitor
        button = self.wtree.get_widget("but_inhib")
        button.connect("clicked", self.on_inhib)
        button.set_sensitive(False)

        # button simul
        button = self.wtree.get_widget("but_simul")
        button.connect("clicked", self.on_simul)

        # button chrono
        button = self.wtree.get_widget("but_chrono")
        button.connect("clicked", self.on_chrono)

        # register
        parent.win_register(self)
        self.parent = parent

        # display
        self.window.show_all()
        #gtk.main()

    def on_destroy(self, widget):
        """
        when leaving the window
        """
        self.clean_subwin()
        if self.simul:
            self.simul.on_destroy(None)
        self.win_remove(self)

    def destroy(self):
        """
        when leaving the window
        """
        self.clean_subwin()
        if self.simul:
            self.simul.on_destroy(None)
        if self.window:
            self.window.destroy()

    def build_button_frame(self, state_list, group = None):
        """
        ???
        """
        template = pkg_resources.resource_filename(
            __name__,
            "../chart_glade/button_frame.glade"
        )
        wtree = gtk.glade.XML(template)
        wid = wtree.get_widget("button_frame")
        frame = wid.get_child()
        wid.remove(frame)

        #textview
        tname = 'tw'+str(self.compt)
        text = wtree.get_widget("textview")
        text.set_name(tname)

        text_buffer = text.get_buffer()
        str_state = '\n'
        for state in state_list :
            str_state += state + '\t'
        str_state += '\n'
        text_buffer.set_text(str_state)
        text.set_editable(False)

        self.textview_list.append(state_list)

        #radio button
        rname = 'rb' + str(self.compt)
        rbut = wtree.get_widget("rButton")
        rbut.set_name(rname)

        if group:
            rbut.set_group(group)
        else :
            self.button_group = rbut
            rbut.set_active(True)

        rbut.connect("toggled", self.rb_callback, self.compt)
        self.compt += 1
        return frame

    def rb_callback(self, widget, id_frame):
        """
        radio button call_back
        """
        self.choice = id_frame

    def on_save(self, widget, all_sol=True):
        """
        open a window to save as xml file
        @param all_sol : boolean - True if we take all solutions
        """
        choice = gtk.FileChooserDialog("Save solution", None ,
                                       gtk.FILE_CHOOSER_ACTION_SAVE,
                                       ( gtk.STOCK_CANCEL,
                                         gtk.RESPONSE_CANCEL,
                                         gtk.STOCK_SAVE,
                                         gtk.RESPONSE_OK))
        choice.set_default_response(gtk.RESPONSE_OK)

        #add a filter to see only xml files for biochart (*.txt)
        ffilter = gtk.FileFilter()
        ffilter.set_name("txt files")
        ffilter.add_pattern("*.txt")
        choice.add_filter(ffilter)

        #add a filter to see all
        no_filter = gtk.FileFilter()
        no_filter.set_name("all")
        no_filter.add_pattern("*")
        choice.add_filter(no_filter)

        response = choice.run()
        if response == gtk.RESPONSE_OK:
            self.create_solution_file(choice.get_filename(), all_sol)
        elif response == gtk.RESPONSE_CANCEL:
            pass
            #print 'Closed, no files selected'
        choice.destroy()

    def create_solution_file(self, sol_file, all_bool):
        """
        make a xml file with the current model
        """
        if all_bool:
            init_places = self.l_solutions
            input_clocks = self.l_icseq
        else :
            init_places = [self.l_solutions[self.choice]]
            input_clocks = [self.l_icseq[self.choice]]

        sfile = open(sol_file,'w')
        for i in range(len(init_places)):
            spla = init_places[i]
            icseq = input_clocks[i]
            for elemt in spla :
                sfile.write(str(elemt)+'\t')
            for ica in icseq:
                sfile.write('\n' + ica)
            sfile.write('\n')
        sfile.close()

    def on_inhib(self, widget):
        """
        Compute inhibitors
        """
#
#        start_list = self.l_solutions[self.choice]
#        start_p = logical_and(start_list)
#        inv_p = 'not('+self.parent.property+')'
#        mac_list = self.parent.mcla.mac_search(start_p, inv_p, None,
#                                               self.parent.max_step)
#        inhib_list = []
#        for mac in mac_list:
#            im = []
#            for var in mac:
#                if var in start_list:
#                    continue
#                else:
#                    im.append(var)
#            if im:
#                inhib_list.append(im)
#        if len(inhib_list) == 0:
#            cancel_warn("This solution has not inhibitor")
#        else :
#            Inhib_Window(inhib_list, self.emvc, self.reporter, self)
        pass

    def on_simul(self, widget):
        """
        Launch simulation of a solution
        """
        if self.simul:
            self.simul.on_destroy(None)
        #init_places = self.choice
        init_places = self.l_solutions[self.choice]
        input_clocks = self.l_icseq[self.choice]
        self.simul = ChartSimulControler(self.emvc, True, self.reporter,
                                         init_places, input_clocks)

    def on_chrono(self, widget):
        """
        Show a chronogram of a trajectory initialized with solution
        """
        # stand alone simulator
        sim = ChartSimulator()
        sim.build(self.emvc.model, True, self.reporter)

        chrono = ChartChrono.for_trajectory(sim)
        self.win_register(chrono)

        # simulation
        init_places = self.l_solutions[self.choice]
        input_clocks = self.l_icseq[self.choice]
        sim.simul_init_places(init_places)
        # input buffer is any input
        if input_clocks:
            sim.set_act_input_stream(input_clocks)
        i = 0
        while i <= self.parent.max_step :
            chrono.register(sim)
            try:
                sim.simul_step()
                i += 1
            except Exception:
                break
        # clean model
        self.emvc.model.clean()
        self.emvc.model.notify()
        # display
        chrono.display_selected_chrono()

    # sub windows management
    def win_register(self, win):
        """
        sub windows management
        """
        if win.__class__ == ChartSimulControler:
            self.simul = win
        self.aux_win.append(win)

    def win_remove(self, win):
        """
        sub windows management
        """
        if win in self.aux_win:
            self.aux_win.remove(win)
        if win == self.simul:
            self.simul = None

    def clean_subwin(self):
        """
        sub windows management
        """
        for win in self.aux_win:
            win.destroy()
        self.aux_win = []


class CNSWindow(SolutionWindow):
    """
    ???
    """
    def __init__(self, l_solutions, emvc, reporter, parent):

        SolutionWindow.__init__(self, l_solutions, emvc, reporter, parent)

        # info label
        info = self.wtree.get_widget("label_nbsol")
        info.set_text(str(len(l_solutions))+" Minimal Activation Conditions")


class InhibWindow(SolutionWindow):
    """
    ???
    """
    def __init__(self, l_solutions, emvc, reporter, parent):

        SolutionWindow.__init__(self, l_solutions, emvc, reporter, parent)

        # info label
        info = self.wtree.get_widget("label_nbsol")
        info.set_text(str(len(l_solutions)) + " inhibitors")

        # button inhibitor
        button = self.wtree.get_widget("but_inhib")
        button.connect("clicked", self.on_inhib)
        button.set_sensitive(False)



def logical_and(list):
    """
    @return: logical_formula: str - AND of the input list
    """

    if len(list) == 0 :
        return
    elif len(list) == 1 :
        return list[0]
    else :
        logical_formula = ''
        for elemnt in list :
            logical_formula += elemnt + ' and '
        logical_formula = logical_formula[:-5]
#        print logical_formula
        return '('+logical_formula+')'

